@@ -52,6 +52,14 @@ CreateShardedCollectionUtil.shardCollectionWithChunks(sourceCollection, {key: 1}
     {min: {key: 0}, max: {key: MaxKey}, shard: st.shard1.shardName},
 ]);
 
+// The ShardServerCatalogCacheLoader thread may perform deletes in the background on the
+// config.cache.chunks.test.mycoll collection. Its deletes are also eligible to be blocked by the
+// hangWithLockDuringBatchRemove failpoint and will prevent a removeOperation thread from acquiring
+// a WiredTiger write ticket. We wait for it to finish any delete operations by explicitly running
+// the _flushRoutingTableCacheUpdates command.
+assert.commandWorked(
+    txnCoordinator.adminCommand({_flushRoutingTableCacheUpdates: sourceCollection.getFullName()}));
+
 const removeOperationThreads = Array.from({length: kNumWriteTickets}).map(() => {
     return new Thread(function removeOperation(host, dbName, collName, insertLatch) {
         const conn = new Mongo(host);
