@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -70,7 +69,7 @@ void mongo::audit::logQueryAuthzCheck(Client* client,
 void mongo::audit::logUpdateAuthzCheck(Client* client,
                                        const NamespaceString& ns,
                                        const BSONObj& query,
-                                       const BSONObj& updateObj,
+                                       const write_ops::UpdateModification& update,
                                        bool isUpsert,
                                        bool isMulti,
                                        ErrorCodes::Error result) {}
@@ -173,7 +172,4 @@ void mongo::audit::logShardCollection(Client* client,
                                       const BSONObj& keyPattern,
                                       bool unique) {}
 
-void mongo::audit::writeImpersonatedUsersToMetadata(OperationContext* opCtx,
-                                                    BSONObjBuilder* metadata) {}
-
 #endif
