@@ -53,9 +53,10 @@
 #include "mongo/db/jsobj.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/rpc/factory.h"
-#include "mongo/rpc/metadata/client_metadata_ismaster.h"
+#include "mongo/rpc/metadata/client_metadata.h"
 #include "mongo/rpc/op_msg_rpc_impls.h"
 #include "mongo/rpc/protocol.h"
+#include "mongo/rpc/rewrite_state_change_errors.h"
 #include "mongo/rpc/write_concern_error_detail.h"
 #include "mongo/s/stale_exception.h"
 #include "mongo/util/fail_point_service.h"
@@ -476,6 +477,11 @@ bool CommandHelpers::shouldActivateFailCommandFailPoint(const BSONObj& data,
     if (cmd->getName() == "configureFailPoint"_sd)  // Banned even if in failCommands.
         return false;
 
+    auto appName = StringData();
+    if (auto clientMetadata = ClientMetadata::get(client)) {
+        appName = clientMetadata->getApplicationName();
+    }
+
     if (data.hasField("threadName") &&
         (client->desc() !=
          data.getStringField(
@@ -483,12 +489,8 @@ bool CommandHelpers::shouldActivateFailCommandFailPoint(const BSONObj& data,
         return false;
     }
 
-    if (data.hasField("appName")) {
-        const auto& clientMetadata = ClientMetadataIsMasterState::get(client).getClientMetadata();
-        if (clientMetadata &&
-            clientMetadata.get().getApplicationName() != data.getStringField("appName")) {
-            return false;  // only activate failpoint on connection with a certain appName
-        }
+    if (data.hasField("appName") && (appName != data.getStringField("appName"))) {
+        return false;  // only activate failpoint on connection with a certain appName
     }
 
     if (client->session() && (client->session()->getTags() & transport::Session::kInternalClient)) {
@@ -521,7 +523,7 @@ void CommandHelpers::evaluateFailCommandFailPoint(OperationContext* opCtx,
     long long errorCode, blockTimeMS;
     const Command* cmd = invocation->definition();
 
-    MONGO_FAIL_POINT_BLOCK_IF(failCommand, data, [&](const BSONObj& data) {
+    MONGO_FAIL_POINT_BLOCK_IF(failCommand, scoped, [&](const BSONObj& data) {
         closeConnection = data.hasField("closeConnection") &&
             bsonExtractBooleanField(data, "closeConnection", &closeConnection).isOK() &&
             closeConnection;
@@ -535,6 +537,13 @@ void CommandHelpers::evaluateFailCommandFailPoint(OperationContext* opCtx,
         return shouldActivateFailCommandFailPoint(data, invocation, opCtx->getClient()) &&
             (closeConnection || blockConnection || hasErrorCode);
     }) {
+        const BSONObj& data = scoped.getData();
+        // State change codes are rewritten on the way out of a `mongos`
+        // server. Errors injected via failpoint manipulation are normally
+        // exempt from this. However, we provide an override option so they
+        // can be made subject to rewriting if that's really necessary.
+        if (bool b; !bsonExtractBooleanField(data, "allowRewriteStateChange", &b).isOK() || !b)
+            rpc::RewriteStateChangeErrors::setEnabled(opCtx, false);
         if (closeConnection) {
             opCtx->getClient()->session()->end();
             log() << "Failing command '" << cmd->getName()
@@ -577,10 +586,8 @@ void CommandHelpers::handleMarkKillOnClientDisconnect(OperationContext* opCtx,
 
     MONGO_FAIL_POINT_BLOCK_IF(
         waitInCommandMarkKillOnClientDisconnect, options, [&](const BSONObj& obj) {
-            const auto& clientMetadata =
-                ClientMetadataIsMasterState::get(opCtx->getClient()).getClientMetadata();
-
-            return clientMetadata && (clientMetadata->getApplicationName() == obj["appName"].str());
+            auto md = ClientMetadata::get(opCtx->getClient());
+            return md && (md->getApplicationName() == obj["appName"].str());
         }) {
         MONGO_FAIL_POINT_PAUSE_WHILE_SET_OR_INTERRUPTED(opCtx,
                                                         waitInCommandMarkKillOnClientDisconnect);
