@@ -140,6 +140,14 @@ public:
             subObjBuilder.doneFast();
         }
 
+        {
+            BSONObjBuilder subObjBuilder(builder.subobjStart("mounts"_sd));
+            processStatusErrors(
+                procparser::parseProcSelfMountStatsFile("/proc/self/mountinfo"_sd, &subObjBuilder),
+                &subObjBuilder);
+            subObjBuilder.doneFast();
+        }
+
         {
             BSONObjBuilder subObjBuilder(builder.subobjStart("vmstat"_sd));
             processStatusErrors(
