@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -33,14 +32,11 @@
 #include "mongo/db/field_parser.h"
 #include "mongo/db/geo/shapes.h"
 #include "mongo/db/jsobj.h"
-#include "mongo/util/mongoutils/str.h"
+#include "mongo/util/str.h"
 
 #include <algorithm>  // for max()
 #include <iostream>
 
-// So we can get at the str namespace.
-using namespace mongoutils;
-
 namespace mongo {
 
 using std::stringstream;
@@ -671,20 +667,14 @@ Status GeoHashConverter::parseParameters(const BSONObj& paramDoc,
     if (params->bits < 1 || params->bits > 32) {
         return Status(ErrorCodes::InvalidOptions,
                       str::stream() << "bits for hash must be > 0 and <= 32, "
-                                    << "but "
-                                    << params->bits
-                                    << " bits were specified");
+                                    << "but " << params->bits << " bits were specified");
     }
 
     const bool rangeValid = params->min < params->max;
     if (!rangeValid || std::isinf(params->min) || std::isinf(params->max)) {
         return Status(ErrorCodes::InvalidOptions,
                       str::stream() << "region for hash must be valid and have positive area, "
-                                    << "but ["
-                                    << params->min
-                                    << ", "
-                                    << params->max
-                                    << "] "
+                                    << "but [" << params->min << ", " << params->max << "] "
                                     << "was specified");
     }
 
@@ -693,8 +683,8 @@ Status GeoHashConverter::parseParameters(const BSONObj& paramDoc,
     const bool scalingValid = params->scaling > 0;
     if (!scalingValid || std::isinf(params->scaling)) {
         return Status(ErrorCodes::InvalidOptions,
-                      str::stream() << "range [" << params->min << ", " << params->max
-                                    << "] is too small.");
+                      str::stream()
+                          << "range [" << params->min << ", " << params->max << "] is too small.");
     }
 
     return Status::OK();
@@ -787,8 +777,7 @@ GeoHash GeoHashConverter::hash(const BSONObj& o, const BSONObj* src) const {
 GeoHash GeoHashConverter::hash(double x, double y) const {
     uassert(16433,
             str::stream() << "point not in interval of [ " << _params.min << ", " << _params.max
-                          << " ]"
-                          << causedBy(BSON_ARRAY(x << y).toString()),
+                          << " ]" << causedBy(BSON_ARRAY(x << y).toString()),
             x <= _params.max && x >= _params.min && y <= _params.max && y >= _params.min);
 
     return GeoHash(convertToHashScale(x), convertToHashScale(y), _params.bits);
