@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -36,8 +35,9 @@
 #include "mongo/db/key_generator.h"
 #include "mongo/db/keys_collection_cache.h"
 #include "mongo/db/keys_collection_document.h"
+#include "mongo/db/keys_collection_manager_gen.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/stdx/functional.h"
-#include "mongo/stdx/mutex.h"
 #include "mongo/stdx/thread.h"
 #include "mongo/util/concurrency/notification.h"
 #include "mongo/util/duration.h"
@@ -49,8 +49,6 @@ class LogicalTime;
 class ServiceContext;
 class KeysCollectionClient;
 
-extern int KeysRotationIntervalSec;
-
 namespace keys_collection_manager_util {
 
 /**
@@ -69,7 +67,6 @@ Milliseconds howMuchSleepNeedFor(const LogicalTime& currentTime,
  */
 class KeysCollectionManager {
 public:
-    static const Seconds kKeyValidInterval;
     static const std::string kKeyManagerPurposeString;
 
     KeysCollectionManager(std::string purpose,
@@ -183,7 +180,8 @@ private:
                                 std::string threadName,
                                 Milliseconds refreshInterval);
 
-        stdx::mutex _mutex;  // protects all the member variables below.
+        // protects all the member variables below.
+        Mutex _mutex = MONGO_MAKE_LATCH("PeriodicRunner::_mutex");
         std::shared_ptr<Notification<void>> _refreshRequest;
         stdx::condition_variable _refreshNeededCV;
 
