@@ -375,6 +375,14 @@ public:
         return _alwaysInterruptAtStepDownOrUp.load();
     }
 
+    /**
+     * Sets that this operation should ignore interruption except for replication state change. Can
+     * only be called by the thread executing this on behalf of this OperationContext.
+     */
+    void setIgnoreInterruptsExceptForReplStateChange(bool target) {
+        _ignoreInterruptsExceptForReplStateChange = target;
+    }
+
     /**
      * Clears metadata associated with a multi-document transaction.
      */
@@ -531,10 +539,13 @@ private:
     bool _writesAreReplicated = true;
     bool _shouldParticipateInFlowControl = true;
     bool _inMultiDocumentTransaction = false;
+    bool _ignoreInterruptsExceptForReplStateChange = false;
 
     // If true, this OpCtx will get interrupted during replica set stepUp and stepDown, regardless
     // of what locks it's taken.
     AtomicWord<bool> _alwaysInterruptAtStepDownOrUp{false};
+
+    AtomicWord<bool> _killRequestedForReplStateChange{false};
 };
 
 namespace repl {
