@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -32,12 +31,11 @@
 
 #include <vector>
 
-#include "mongo/base/disallow_copying.h"
 #include "mongo/base/status.h"
 #include "mongo/db/query/canonical_query.h"
 #include "mongo/db/query/index_entry.h"
 #include "mongo/db/query/index_tag.h"
-#include "mongo/db/query/query_knobs.h"
+#include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/stdx/unordered_map.h"
 
 namespace mongo {
@@ -77,7 +75,8 @@ struct PlanEnumeratorParams {
  * predicate information to make better decisions about what indices are best.
  */
 class PlanEnumerator {
-    MONGO_DISALLOW_COPYING(PlanEnumerator);
+    PlanEnumerator(const PlanEnumerator&) = delete;
+    PlanEnumerator& operator=(const PlanEnumerator&) = delete;
 
 public:
     /**
