@@ -79,8 +79,13 @@ bool filterMatches(const BSONObj& testFilter,
     if (!statusWithMatcher.isOK()) {
         return false;
     }
-    const std::unique_ptr<MatchExpression> root = std::move(statusWithMatcher.getValue());
+    std::unique_ptr<MatchExpression> root = std::move(statusWithMatcher.getValue());
     CanonicalQuery::sortTree(root.get());
+    if (root->matchType() == mongo::MatchExpression::NOT) {
+        // Ideally we would optimize() everything, but some of the tests depend on structural
+        // equivalence of single-arg $or expressions.
+        root = MatchExpression::optimize(std::move(root));
+    }
     std::unique_ptr<MatchExpression> trueFilter(trueFilterNode->filter->shallowClone());
     CanonicalQuery::sortTree(trueFilter.get());
     return trueFilter->equivalent(root.get());
