@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -47,7 +46,6 @@
 #include "mongo/db/exec/fetch.h"
 #include "mongo/db/exec/geo_near.h"
 #include "mongo/db/exec/index_scan.h"
-#include "mongo/db/exec/keep_mutations.h"
 #include "mongo/db/exec/limit.h"
 #include "mongo/db/exec/merge_sort.h"
 #include "mongo/db/exec/or.h"
@@ -69,7 +67,7 @@ using std::unique_ptr;
 using stdx::make_unique;
 
 PlanStage* buildStages(OperationContext* opCtx,
-                       Collection* collection,
+                       const Collection* collection,
                        const CanonicalQuery& cq,
                        const QuerySolution& qsol,
                        const QuerySolutionNode* root,
@@ -78,15 +76,13 @@ PlanStage* buildStages(OperationContext* opCtx,
         case STAGE_COLLSCAN: {
             const CollectionScanNode* csn = static_cast<const CollectionScanNode*>(root);
             CollectionScanParams params;
-            params.collection = collection;
             params.tailable = csn->tailable;
             params.shouldTrackLatestOplogTimestamp = csn->shouldTrackLatestOplogTimestamp;
             params.assertMinTsHasNotFallenOffOplog = csn->assertMinTsHasNotFallenOffOplog;
             params.direction = (csn->direction == 1) ? CollectionScanParams::FORWARD
                                                      : CollectionScanParams::BACKWARD;
-            params.maxScan = csn->maxScan;
             params.shouldWaitForOplogVisibility = csn->shouldWaitForOplogVisibility;
-            return new CollectionScan(opCtx, params, ws, csn->filter.get());
+            return new CollectionScan(opCtx, collection, params, ws, csn->filter.get());
         }
         case STAGE_IXSCAN: {
             const IndexScanNode* ixn = static_cast<const IndexScanNode*>(root);
@@ -96,22 +92,25 @@ PlanStage* buildStages(OperationContext* opCtx,
                 return nullptr;
             }
 
-            IndexScanParams params;
-
-            params.descriptor =
-                collection->getIndexCatalog()->findIndexByName(opCtx, ixn->index.name);
-            invariant(params.descriptor,
-                      str::stream() << "Namespace: " << collection->ns().toString()
-                                    << ", CanonicalQuery: "
-                                    << cq.toStringShort()
-                                    << ", IndexEntry: "
-                                    << ixn->index.toString());
+            auto descriptor = collection->getIndexCatalog()->findIndexByName(
+                opCtx, ixn->index.identifier.catalogName);
+            invariant(descriptor,
+                      str::stream() << "Namespace: " << collection->ns()
+                                    << ", CanonicalQuery: " << cq.toStringShort()
+                                    << ", IndexEntry: " << ixn->index.toString());
 
+            // We use the node's internal name, keyPattern and multikey details here. For $**
+            // indexes, these may differ from the information recorded in the index's descriptor.
+            IndexScanParams params{descriptor,
+                                   ixn->index.identifier.catalogName,
+                                   ixn->index.keyPattern,
+                                   ixn->index.multikeyPaths,
+                                   ixn->index.multikey};
             params.bounds = ixn->bounds;
             params.direction = ixn->direction;
-            params.maxScan = ixn->maxScan;
             params.addKeyMetadata = ixn->addKeyMetadata;
-            return new IndexScan(opCtx, params, ws, ixn->filter.get());
+            params.shouldDedup = ixn->shouldDedup;
+            return new IndexScan(opCtx, std::move(params), ws, ixn->filter.get());
         }
         case STAGE_FETCH: {
             const FetchNode* fn = static_cast<const FetchNode*>(root);
@@ -128,7 +127,6 @@ PlanStage* buildStages(OperationContext* opCtx,
                 return nullptr;
             }
             SortStageParams params;
-            params.collection = collection;
             params.pattern = sn->pattern;
             params.limit = sn->limit;
             return new SortStage(opCtx, params, ws, childStage);
@@ -143,31 +141,38 @@ PlanStage* buildStages(OperationContext* opCtx,
             return new SortKeyGeneratorStage(
                 opCtx, childStage, ws, keyGenNode->sortSpec, cq.getCollator());
         }
-        case STAGE_PROJECTION: {
-            const ProjectionNode* pn = static_cast<const ProjectionNode*>(root);
-            PlanStage* childStage = buildStages(opCtx, collection, cq, qsol, pn->children[0], ws);
+        case STAGE_PROJECTION_DEFAULT: {
+            auto pn = static_cast<const ProjectionNodeDefault*>(root);
+            unique_ptr<PlanStage> childStage{
+                buildStages(opCtx, collection, cq, qsol, pn->children[0], ws)};
             if (nullptr == childStage) {
                 return nullptr;
             }
-
-            ProjectionStageParams params;
-            params.projObj = pn->projection;
-            params.collator = cq.getCollator();
-
-            // Stuff the right data into the params depending on what proj impl we use.
-            if (ProjectionNode::DEFAULT == pn->projType) {
-                params.fullExpression = pn->fullExpression;
-                params.projImpl = ProjectionStageParams::NO_FAST_PATH;
-            } else if (ProjectionNode::COVERED_ONE_INDEX == pn->projType) {
-                params.projImpl = ProjectionStageParams::COVERED_ONE_INDEX;
-                params.coveredKeyObj = pn->coveredKeyObj;
-                invariant(!pn->coveredKeyObj.isEmpty());
-            } else {
-                invariant(ProjectionNode::SIMPLE_DOC == pn->projType);
-                params.projImpl = ProjectionStageParams::SIMPLE_DOC;
+            return new ProjectionStageDefault(opCtx,
+                                              pn->projection,
+                                              ws,
+                                              std::move(childStage),
+                                              pn->fullExpression,
+                                              cq.getCollator());
+        }
+        case STAGE_PROJECTION_COVERED: {
+            auto pn = static_cast<const ProjectionNodeCovered*>(root);
+            unique_ptr<PlanStage> childStage{
+                buildStages(opCtx, collection, cq, qsol, pn->children[0], ws)};
+            if (nullptr == childStage) {
+                return nullptr;
             }
-
-            return new ProjectionStage(opCtx, params, ws, childStage);
+            return new ProjectionStageCovered(
+                opCtx, pn->projection, ws, std::move(childStage), pn->coveredKeyObj);
+        }
+        case STAGE_PROJECTION_SIMPLE: {
+            auto pn = static_cast<const ProjectionNodeSimple*>(root);
+            unique_ptr<PlanStage> childStage{
+                buildStages(opCtx, collection, cq, qsol, pn->children[0], ws)};
+            if (nullptr == childStage) {
+                return nullptr;
+            }
+            return new ProjectionStageSimple(opCtx, pn->projection, ws, std::move(childStage));
         }
         case STAGE_LIMIT: {
             const LimitNode* ln = static_cast<const LimitNode*>(root);
@@ -187,7 +192,7 @@ PlanStage* buildStages(OperationContext* opCtx,
         }
         case STAGE_AND_HASH: {
             const AndHashNode* ahn = static_cast<const AndHashNode*>(root);
-            auto ret = make_unique<AndHashStage>(opCtx, ws, collection);
+            auto ret = make_unique<AndHashStage>(opCtx, ws);
             for (size_t i = 0; i < ahn->children.size(); ++i) {
                 PlanStage* childStage =
                     buildStages(opCtx, collection, cq, qsol, ahn->children[i], ws);
@@ -213,7 +218,7 @@ PlanStage* buildStages(OperationContext* opCtx,
         }
         case STAGE_AND_SORTED: {
             const AndSortedNode* asn = static_cast<const AndSortedNode*>(root);
-            auto ret = make_unique<AndSortedStage>(opCtx, ws, collection);
+            auto ret = make_unique<AndSortedStage>(opCtx, ws);
             for (size_t i = 0; i < asn->children.size(); ++i) {
                 PlanStage* childStage =
                     buildStages(opCtx, collection, cq, qsol, asn->children[i], ws);
@@ -230,7 +235,7 @@ PlanStage* buildStages(OperationContext* opCtx,
             params.dedup = msn->dedup;
             params.pattern = msn->sort;
             params.collator = cq.getCollator();
-            auto ret = make_unique<MergeSortStage>(opCtx, params, ws, collection);
+            auto ret = make_unique<MergeSortStage>(opCtx, params, ws);
             for (size_t i = 0; i < msn->children.size(); ++i) {
                 PlanStage* childStage =
                     buildStages(opCtx, collection, cq, qsol, msn->children[i], ws);
@@ -251,12 +256,11 @@ PlanStage* buildStages(OperationContext* opCtx,
             params.addPointMeta = node->addPointMeta;
             params.addDistMeta = node->addDistMeta;
 
-            IndexDescriptor* twoDIndex =
-                collection->getIndexCatalog()->findIndexByName(opCtx, node->index.name);
+            const IndexDescriptor* twoDIndex = collection->getIndexCatalog()->findIndexByName(
+                opCtx, node->index.identifier.catalogName);
             invariant(twoDIndex);
 
-            GeoNear2DStage* nearStage =
-                new GeoNear2DStage(params, opCtx, ws, collection, twoDIndex);
+            GeoNear2DStage* nearStage = new GeoNear2DStage(params, opCtx, ws, twoDIndex);
 
             return nearStage;
         }
@@ -270,19 +274,19 @@ PlanStage* buildStages(OperationContext* opCtx,
             params.addPointMeta = node->addPointMeta;
             params.addDistMeta = node->addDistMeta;
 
-            IndexDescriptor* s2Index =
-                collection->getIndexCatalog()->findIndexByName(opCtx, node->index.name);
+            const IndexDescriptor* s2Index = collection->getIndexCatalog()->findIndexByName(
+                opCtx, node->index.identifier.catalogName);
             invariant(s2Index);
 
-            return new GeoNear2DSphereStage(params, opCtx, ws, collection, s2Index);
+            return new GeoNear2DSphereStage(params, opCtx, ws, s2Index);
         }
         case STAGE_TEXT: {
             const TextNode* node = static_cast<const TextNode*>(root);
-            IndexDescriptor* desc =
-                collection->getIndexCatalog()->findIndexByName(opCtx, node->index.name);
+            const IndexDescriptor* desc = collection->getIndexCatalog()->findIndexByName(
+                opCtx, node->index.identifier.catalogName);
             invariant(desc);
-            const FTSAccessMethod* fam =
-                static_cast<FTSAccessMethod*>(collection->getIndexCatalog()->getIndex(desc));
+            const FTSAccessMethod* fam = static_cast<const FTSAccessMethod*>(
+                collection->getIndexCatalog()->getEntry(desc)->accessMethod());
             invariant(fam);
 
             TextStageParams params(fam->getSpec());
@@ -299,23 +303,16 @@ PlanStage* buildStages(OperationContext* opCtx,
         }
         case STAGE_SHARDING_FILTER: {
             const ShardingFilterNode* fn = static_cast<const ShardingFilterNode*>(root);
-            PlanStage* childStage = buildStages(opCtx, collection, cq, qsol, fn->children[0], ws);
+            std::unique_ptr<PlanStage> childStage{
+                buildStages(opCtx, collection, cq, qsol, fn->children[0], ws)};
             if (nullptr == childStage) {
                 return nullptr;
             }
+
+            auto css = CollectionShardingState::get(opCtx, collection->ns());
+            auto scopedCollMetadata = css->getOrphansFilter(opCtx);
             return new ShardFilterStage(
-                opCtx,
-                CollectionShardingState::get(opCtx, collection->ns())->getOrphansFilter(opCtx),
-                ws,
-                childStage);
-        }
-        case STAGE_KEEP_MUTATIONS: {
-            const KeepMutationsNode* km = static_cast<const KeepMutationsNode*>(root);
-            PlanStage* childStage = buildStages(opCtx, collection, cq, qsol, km->children[0], ws);
-            if (nullptr == childStage) {
-                return nullptr;
-            }
-            return new KeepMutationsStage(opCtx, km->filter.get(), ws, childStage);
+                opCtx, std::move(scopedCollMetadata), ws, childStage.release());
         }
         case STAGE_DISTINCT_SCAN: {
             const DistinctNode* dn = static_cast<const DistinctNode*>(root);
@@ -325,15 +322,22 @@ PlanStage* buildStages(OperationContext* opCtx,
                 return nullptr;
             }
 
-            DistinctParams params;
+            auto descriptor = collection->getIndexCatalog()->findIndexByName(
+                opCtx, dn->index.identifier.catalogName);
+            invariant(descriptor);
 
-            params.descriptor =
-                collection->getIndexCatalog()->findIndexByName(opCtx, dn->index.name);
-            invariant(params.descriptor);
-            params.direction = dn->direction;
+            // We use the node's internal name, keyPattern and multikey details here. For $**
+            // indexes, these may differ from the information recorded in the index's descriptor.
+            DistinctParams params{descriptor,
+                                  dn->index.identifier.catalogName,
+                                  dn->index.keyPattern,
+                                  dn->index.multikeyPaths,
+                                  dn->index.multikey};
+
+            params.scanDirection = dn->direction;
             params.bounds = dn->bounds;
             params.fieldNo = dn->fieldNo;
-            return new DistinctScan(opCtx, params, ws);
+            return new DistinctScan(opCtx, std::move(params), ws);
         }
         case STAGE_COUNT_SCAN: {
             const CountScanNode* csn = static_cast<const CountScanNode*>(root);
@@ -343,17 +347,23 @@ PlanStage* buildStages(OperationContext* opCtx,
                 return nullptr;
             }
 
-            CountScanParams params;
+            auto descriptor = collection->getIndexCatalog()->findIndexByName(
+                opCtx, csn->index.identifier.catalogName);
+            invariant(descriptor);
+
+            // We use the node's internal name, keyPattern and multikey details here. For $**
+            // indexes, these may differ from the information recorded in the index's descriptor.
+            CountScanParams params{descriptor,
+                                   csn->index.identifier.catalogName,
+                                   csn->index.keyPattern,
+                                   csn->index.multikeyPaths,
+                                   csn->index.multikey};
 
-            params.descriptor =
-                collection->getIndexCatalog()->findIndexByName(opCtx, csn->index.name);
-            invariant(params.descriptor);
             params.startKey = csn->startKey;
             params.startKeyInclusive = csn->startKeyInclusive;
             params.endKey = csn->endKey;
             params.endKeyInclusive = csn->endKeyInclusive;
-
-            return new CountScan(opCtx, params, ws);
+            return new CountScan(opCtx, std::move(params), ws);
         }
         case STAGE_ENSURE_SORTED: {
             const EnsureSortedNode* esn = static_cast<const EnsureSortedNode*>(root);
@@ -367,22 +377,20 @@ PlanStage* buildStages(OperationContext* opCtx,
         case STAGE_CHANGE_STREAM_PROXY:
         case STAGE_COUNT:
         case STAGE_DELETE:
-        case STAGE_NOTIFY_DELETE:
         case STAGE_EOF:
-        case STAGE_GROUP:
         case STAGE_IDHACK:
-        case STAGE_INDEX_ITERATOR:
         case STAGE_MULTI_ITERATOR:
         case STAGE_MULTI_PLAN:
-        case STAGE_OPLOG_START:
         case STAGE_PIPELINE_PROXY:
         case STAGE_QUEUED_DATA:
+        case STAGE_RECORD_STORE_FAST_COUNT:
         case STAGE_SUBPLAN:
-        case STAGE_TEXT_OR:
         case STAGE_TEXT_MATCH:
+        case STAGE_TEXT_OR:
+        case STAGE_TRIAL:
         case STAGE_UNKNOWN:
         case STAGE_UPDATE: {
-            mongoutils::str::stream ss;
+            str::stream ss;
             root->appendToString(&ss, 0);
             string nodeStr(ss);
             warning() << "Can't build exec tree for node " << nodeStr << endl;
@@ -393,7 +401,7 @@ PlanStage* buildStages(OperationContext* opCtx,
 
 // static (this one is used for Cached and MultiPlanStage)
 bool StageBuilder::build(OperationContext* opCtx,
-                         Collection* collection,
+                         const Collection* collection,
                          const CanonicalQuery& cq,
                          const QuerySolution& solution,
                          WorkingSet* wsIn,
