@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -41,9 +40,10 @@
 #include "mongo/db/repl/oplog_buffer.h"
 #include "mongo/db/repl/oplog_entry.h"
 #include "mongo/db/repl/replication_consistency_markers.h"
+#include "mongo/db/repl/session_update_tracker.h"
 #include "mongo/db/repl/storage_interface.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/stdx/functional.h"
-#include "mongo/stdx/mutex.h"
 #include "mongo/util/concurrency/thread_pool.h"
 
 namespace mongo {
@@ -72,30 +72,6 @@ public:
                                                      WorkerMultikeyPathInfo* workerMultikeyPathInfo,
                                                      const bool isDataConsistent)>;
 
-    /**
-     * Maximum number of operations in each batch that can be applied using multiApply().
-     */
-    static AtomicInt32 replBatchLimitOperations;
-
-    /**
-     * Lower bound of batch limit size (in bytes) returned by calculateBatchLimitBytes().
-     */
-    static AtomicInt32 replBatchLimitBytes;
-
-    /**
-     * Calculates batch limit size (in bytes) using the maximum capped collection size of the oplog
-     * size.
-     * Batches are limited to 10% of the oplog.
-     */
-    static std::size_t calculateBatchLimitBytes(OperationContext* opCtx,
-                                                StorageInterface* storageInterface);
-
-    /**
-     * Creates thread pool for writer tasks.
-     */
-    static std::unique_ptr<ThreadPool> makeWriterPool();
-    static std::unique_ptr<ThreadPool> makeWriterPool(int threadCount);
-
     /**
      * Applies the operation that is in param o.
      * Functions for applying operations/commands and increment server status counters may
@@ -104,7 +80,8 @@ public:
     static Status syncApply(OperationContext* opCtx,
                             const BSONObj& o,
                             OplogApplication::Mode oplogApplicationMode,
-                            const bool isDataConsistent);
+                            const bool isDataConsistent,
+                            boost::optional<Timestamp> stableTimestampForRecovery);
 
     /**
      *
@@ -121,11 +98,6 @@ public:
              MultiSyncApplyFunc func,
              ThreadPool* writerPool,
              const OplogApplier::Options& options);
-    SyncTail(OplogApplier::Observer* observer,
-             ReplicationConsistencyMarkers* consistencyMarkers,
-             StorageInterface* storageInterface,
-             MultiSyncApplyFunc func,
-             ThreadPool* writerPool);
     virtual ~SyncTail();
 
     /**
@@ -138,7 +110,9 @@ public:
      * Retrieves operations from the OplogBuffer in batches that will be applied in parallel using
      * multiApply().
      */
-    void oplogApplication(OplogBuffer* oplogBuffer, ReplicationCoordinator* replCoord);
+    void oplogApplication(OplogBuffer* oplogBuffer,
+                          OplogApplier::GetNextApplierBatchFn getNextApplierBatchFn,
+                          ReplicationCoordinator* replCoord);
 
     /**
      * Shuts down oplogApplication() processing.
@@ -232,36 +206,28 @@ public:
 
     using BatchLimits = OplogApplier::BatchLimits;
 
-    /**
-     * Attempts to pop an OplogEntry off the BGSync queue and add it to ops.
-     *
-     * Returns true if the (possibly empty) batch in ops should be ended and a new one started.
-     * If ops is empty on entry and nothing can be added yet, will wait up to a second before
-     * returning true.
-     */
-    bool tryPopAndWaitForMore(OperationContext* opCtx,
-                              OplogBuffer* oplogBuffer,
-                              OpQueue* ops,
-                              const BatchLimits& limits);
-
     /**
      * Fetch a single document referenced in the operation from the sync source.
+     *
+     * The sync source is specified at construction in
+     * OplogApplier::Options::missingDocumentSourceForInitialSync.
      */
     virtual BSONObj getMissingDoc(OperationContext* opCtx, const OplogEntry& oplogEntry);
 
     /**
      * If an update fails, fetches the missing document and inserts it into the local collection.
      *
-     * Returns true if the document was fetched and inserted successfully.
+     * Calls OplogApplier::Observer::onMissingDocumentsFetchedAndInserted() if the document was
+     * fetched and inserted successfully.
      */
-    virtual bool fetchAndInsertMissingDocument(OperationContext* opCtx,
+    virtual void fetchAndInsertMissingDocument(OperationContext* opCtx,
                                                const OplogEntry& oplogEntry);
 
-    void setHostname(const std::string& hostname);
-
     /**
      * Applies a batch of oplog entries by writing the oplog entries to the local oplog and then
-     * using a set of threads to apply the operations.
+     * using a set of threads to apply the operations. It will only apply (but will
+     * still write to the oplog) oplog entries with a timestamp greater than or equal to the
+     * beginApplyingTimestamp.
      *
      * If the batch application is successful, returns the optime of the last op applied, which
      * should be the last op in the batch.
@@ -273,20 +239,30 @@ public:
      */
     StatusWith<OpTime> multiApply(OperationContext* opCtx, MultiApplier::Operations ops);
 
-private:
-    /**
-     * Pops the operation at the front of the OplogBuffer.
-     * Updates stats on BackgroundSync.
-     */
-    void _consume(OperationContext* opCtx, OplogBuffer* oplogBuffer);
+    void fillWriterVectors(OperationContext* opCtx,
+                           MultiApplier::Operations* ops,
+                           std::vector<MultiApplier::OperationPtrs>* writerVectors,
+                           std::vector<MultiApplier::Operations>* derivedOps) noexcept;
 
+private:
     class OpQueueBatcher;
 
-    void _oplogApplication(OplogBuffer* oplogBuffer,
-                           ReplicationCoordinator* replCoord,
-                           OpQueueBatcher* batcher) noexcept;
+    void _oplogApplication(ReplicationCoordinator* replCoord, OpQueueBatcher* batcher) noexcept;
 
-    std::string _hostname;
+    void _fillWriterVectors(OperationContext* opCtx,
+                            MultiApplier::Operations* ops,
+                            std::vector<MultiApplier::OperationPtrs>* writerVectors,
+                            std::vector<MultiApplier::Operations>* derivedOps,
+                            SessionUpdateTracker* sessionUpdateTracker) noexcept;
+
+    /**
+     * Doles out all the work to the writer pool threads. Does not modify writerVectors, but passes
+     * non-const pointers to inner vectors into func.
+     */
+    void _applyOps(std::vector<MultiApplier::OperationPtrs>& writerVectors,
+                   std::vector<Status>* statusVector,
+                   std::vector<WorkerMultikeyPathInfo>* workerMultikeyPathInfo,
+                   const bool isDataConsistent);
 
     OplogApplier::Observer* const _observer;
     ReplicationConsistencyMarkers* const _consistencyMarkers;
@@ -303,15 +279,15 @@ private:
     const OplogApplier::Options _options;
 
     // Protects member data of SyncTail.
-    mutable stdx::mutex _mutex;
+    mutable Mutex _mutex = MONGO_MAKE_LATCH("SyncTail::_mutex");
 
     // Set to true if shutdown() has been called.
     bool _inShutdown = false;
 };
 
-// These free functions are used by the thread pool workers to write ops to the db.
-// They consume the passed in OperationPtrs and callers should not make any assumptions about the
-// state of the container after calling. However, these functions cannot modify the pointed-to
+// This free function is used by the thread pool workers to write ops to the db.
+// This consumes the passed in OperationPtrs and callers should not make any assumptions about the
+// state of the container after calling. However, this function cannot modify the pointed-to
 // operations because the OperationPtrs container contains const pointers.
 Status multiSyncApply(OperationContext* opCtx,
                       MultiApplier::OperationPtrs* ops,
@@ -319,11 +295,5 @@ Status multiSyncApply(OperationContext* opCtx,
                       WorkerMultikeyPathInfo* workerMultikeyPathInfo,
                       const bool isDataConsistent);
 
-Status multiInitialSyncApply(OperationContext* opCtx,
-                             MultiApplier::OperationPtrs* ops,
-                             SyncTail* st,
-                             WorkerMultikeyPathInfo* workerMultikeyPathInfo,
-                             const bool isDataConsistent);
-
 }  // namespace repl
 }  // namespace mongo
