@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -386,5 +385,34 @@ TEST_F(CollectionRangeDeleterTest, MultipleDocumentsInMultipleRangesToClean) {
     ASSERT_FALSE(next(rangeDeleter, 1));
 }
 
+// Tests that we retry on a WriteConflictException.
+TEST_F(CollectionRangeDeleterTest, RetryOnWriteConflictException) {
+    CollectionRangeDeleter rangeDeleter;
+    DBDirectClient dbclient(operationContext());
+
+    dbclient.insert(kNss.toString(), BSON(kShardKey << 1));
+    dbclient.insert(kNss.toString(), BSON(kShardKey << 2));
+    dbclient.insert(kNss.toString(), BSON(kShardKey << 3));
+    ASSERT_EQUALS(3ULL, dbclient.count(kNss.toString(), BSON(kShardKey << LT << 5)));
+
+    std::list<Deletion> ranges;
+    auto deletion = Deletion{ChunkRange(BSON(kShardKey << 0), BSON(kShardKey << 10)), Date_t{}};
+    ranges.emplace_back(std::move(deletion));
+    auto when = rangeDeleter.add(std::move(ranges));
+    ASSERT(when && *when == Date_t{});
+
+    // TODO SERVER-41606: Remove this function when we refactor CollectionRangeDeleter.
+    rangeDeleter.setDoDeletionShouldThrowWriteConflictForTest(true);
+
+    ASSERT_TRUE(next(rangeDeleter, 1));
+    ASSERT_EQUALS(3ULL, dbclient.count(kNss.toString(), BSON(kShardKey << LT << 5)));
+
+    // TODO SERVER-41606: Remove this function when we refactor CollectionRangeDeleter.
+    rangeDeleter.setDoDeletionShouldThrowWriteConflictForTest(false);
+
+    ASSERT_TRUE(next(rangeDeleter, 1));
+    ASSERT_EQUALS(2ULL, dbclient.count(kNss.toString(), BSON(kShardKey << LT << 5)));
+}
+
 }  // namespace
 }  // namespace mongo
