@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -50,20 +49,19 @@
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/config/initial_split_policy.h"
 #include "mongo/db/s/config/sharding_catalog_manager.h"
-#include "mongo/db/sessions_collection.h"
 #include "mongo/s/balancer_configuration.h"
 #include "mongo/s/catalog/type_database.h"
 #include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/catalog_cache.h"
 #include "mongo/s/client/shard_registry.h"
-#include "mongo/s/commands/cluster_commands_helpers.h"
+#include "mongo/s/cluster_commands_helpers.h"
 #include "mongo/s/config_server_client.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/request_types/shard_collection_gen.h"
 #include "mongo/s/shard_util.h"
 #include "mongo/util/log.h"
-#include "mongo/util/mongoutils/str.h"
 #include "mongo/util/scopeguard.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace {
@@ -150,7 +148,7 @@ void validateAndDeduceFullRequestOptions(OperationContext* opCtx,
     // Ensure the namespace is valid.
     uassert(ErrorCodes::IllegalOperation,
             "can't shard system namespaces",
-            !nss.isSystem() || nss == SessionsCollection::kSessionsNamespaceString);
+            !nss.isSystem() || nss == NamespaceString::kLogicalSessionsNamespace);
 
     // Ensure the collation is valid. Currently we only allow the simple collation.
     bool simpleCollationSpecified = false;
@@ -160,8 +158,7 @@ void validateAndDeduceFullRequestOptions(OperationContext* opCtx,
             CollatorFactoryInterface::get(opCtx->getServiceContext())->makeFromBSON(collation));
         uassert(ErrorCodes::BadValue,
                 str::stream() << "The collation for shardCollection must be {locale: 'simple'}, "
-                              << "but found: "
-                              << collation,
+                              << "but found: " << collation,
                 !collator);
         simpleCollationSpecified = true;
     }
@@ -175,8 +172,7 @@ void validateAndDeduceFullRequestOptions(OperationContext* opCtx,
     int numChunks = request->getNumInitialChunks();
     uassert(ErrorCodes::InvalidOptions,
             str::stream() << "numInitialChunks cannot be more than either: "
-                          << maxNumInitialChunksForShards
-                          << ", 8192 * number of shards; or "
+                          << maxNumInitialChunksForShards << ", 8192 * number of shards; or "
                           << maxNumInitialChunksTotal,
             numChunks >= 0 && numChunks <= maxNumInitialChunksForShards &&
                 numChunks <= maxNumInitialChunksTotal);
@@ -305,9 +301,7 @@ void validateShardKeyAgainstExistingIndexes(OperationContext* opCtx,
         bool isUnique = idx["unique"].trueValue();
         uassert(ErrorCodes::InvalidOptions,
                 str::stream() << "can't shard collection '" << nss.ns() << "' with unique index on "
-                              << currentKey
-                              << " and proposed shard key "
-                              << proposedKey
+                              << currentKey << " and proposed shard key " << proposedKey
                               << ". Uniqueness can't be maintained unless shard key is a prefix",
                 !isUnique || shardKeyPattern.isUniqueIndexCompatible(currentKey));
     }
@@ -325,8 +319,7 @@ void validateShardKeyAgainstExistingIndexes(OperationContext* opCtx,
             // per field per collection.
             uassert(ErrorCodes::InvalidOptions,
                     str::stream() << "can't shard collection " << nss.ns()
-                                  << " with hashed shard key "
-                                  << proposedKey
+                                  << " with hashed shard key " << proposedKey
                                   << " because the hashed index uses a non-default seed of "
                                   << idx["seed"].numberInt(),
                     !shardKeyPattern.isHashedPattern() || idx["seed"].eoo() ||
@@ -355,7 +348,7 @@ void validateShardKeyAgainstExistingIndexes(OperationContext* opCtx,
         } else {
             bool isExplicitlyUnique = eqQueryResult["unique"].trueValue();
             BSONObj currKey = eqQueryResult["key"].embeddedObject();
-            bool isCurrentID = str::equals(currKey.firstElementFieldName(), "_id");
+            bool isCurrentID = (currKey.firstElementFieldNameStringData() == "_id");
             uassert(ErrorCodes::InvalidOptions,
                     str::stream() << "can't shard collection " << nss.ns() << ", " << proposedKey
                                   << " index not unique, and unique index explicitly specified",
@@ -440,9 +433,7 @@ void migrateAndFurtherSplitInitialChunks(OperationContext* opCtx,
     auto chunkManager = routingInfo.cm();
 
     // Move and commit each "big chunk" to a different shard.
-    auto nextShardId = [&, indx = 0 ]() mutable {
-        return shardIds[indx++ % shardIds.size()];
-    };
+    auto nextShardId = [&, indx = 0]() mutable { return shardIds[indx++ % shardIds.size()]; };
 
     for (auto chunk : chunkManager->chunks()) {
         const auto shardId = nextShardId();
@@ -507,7 +498,7 @@ void migrateAndFurtherSplitInitialChunks(OperationContext* opCtx,
                     chunkManager->getShardKeyPattern(),
                     chunkManager->getVersion(),
                     ChunkRange(currentChunk->getMin(), currentChunk->getMax()),
-                    subSplits);
+                    &subSplits);
                 if (!splitStatus.isOK()) {
                     warning() << "couldn't split chunk " << redact(currentChunk->toString())
                               << " while sharding collection " << nss.ns()
@@ -533,7 +524,6 @@ void migrateAndFurtherSplitInitialChunks(OperationContext* opCtx,
         }
     }
 }
-
 boost::optional<UUID> getUUIDFromPrimaryShard(OperationContext* opCtx,
                                               const NamespaceString& nss,
                                               const std::shared_ptr<Shard>& primaryShard) {
@@ -556,10 +546,7 @@ boost::optional<UUID> getUUIDFromPrimaryShard(OperationContext* opCtx,
 
     uassert(ErrorCodes::InternalError,
             str::stream() << "expected the primary shard host " << primaryShard->getConnString()
-                          << " for database "
-                          << nss.db()
-                          << " to return an entry for "
-                          << nss.ns()
+                          << " for database " << nss.db() << " to return an entry for " << nss.ns()
                           << " in its listCollections response, but it did not",
             !res.isEmpty());
 
@@ -571,15 +558,12 @@ boost::optional<UUID> getUUIDFromPrimaryShard(OperationContext* opCtx,
     uassert(ErrorCodes::InternalError,
             str::stream() << "expected primary shard to return 'info' field as part of "
                              "listCollections for "
-                          << nss.ns()
-                          << " because the cluster is in featureCompatibilityVersion=3.6, but got "
-                          << res,
+                          << nss.ns() << ", but got " << res,
             !collectionInfo.isEmpty());
 
     uassert(ErrorCodes::InternalError,
             str::stream() << "expected primary shard to return a UUID for collection " << nss.ns()
-                          << " as part of 'info' field but got "
-                          << res,
+                          << " as part of 'info' field but got " << res,
             collectionInfo.hasField("uuid"));
 
     return uassertStatusOK(UUID::parse(collectionInfo["uuid"]));
@@ -672,6 +656,11 @@ public:
                 str::stream() << "sharding not enabled for db " << nss.db(),
                 dbType.getSharded());
 
+        uassert(ErrorCodes::InvalidNamespace,
+                str::stream() << "Namespace too long. Namespace: " << nss
+                              << " Max: " << NamespaceString::MaxNsShardedCollectionLen,
+                nss.size() <= NamespaceString::MaxNsShardedCollectionLen);
+
         // Get variables required throughout this command.
 
         auto proposedKey(request.getKey().getOwned());
@@ -688,8 +677,7 @@ public:
             // Only whitelisted collections in config may be sharded (unless we are in test mode)
             uassert(ErrorCodes::IllegalOperation,
                     "only special collections in the config db may be sharded",
-                    nss == SessionsCollection::kSessionsNamespaceString ||
-                        getTestCommandsEnabled());
+                    nss == NamespaceString::kLogicalSessionsNamespace);
 
             auto configShard = uassertStatusOK(shardRegistry->getShard(opCtx, dbType.getPrimary()));
             auto countCmd = BSON("count" << nss.coll());
@@ -810,8 +798,7 @@ public:
             if (fromMapReduce) {
                 uassert(ErrorCodes::ConflictingOperationInProgress,
                         str::stream() << "Map reduce with sharded output to a new collection found "
-                                      << nss.ns()
-                                      << " to be non-empty which is not supported.",
+                                      << nss.ns() << " to be non-empty which is not supported.",
                         isEmpty);
             }
 
