@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -36,15 +35,15 @@
 #include "mongo/bson/bsonobj.h"
 #include "mongo/client/connection_string.h"
 #include "mongo/db/namespace_string.h"
-#include "mongo/db/query/plan_executor.h"
 #include "mongo/db/repl/optime.h"
 #include "mongo/db/s/migration_chunk_cloner_source.h"
 #include "mongo/db/s/migration_session_id.h"
 #include "mongo/db/s/session_catalog_migration_source.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/s/request_types/move_chunk_request.h"
 #include "mongo/s/shard_key_pattern.h"
+#include "mongo/stdx/condition_variable.h"
 #include "mongo/stdx/memory.h"
-#include "mongo/stdx/mutex.h"
 #include "mongo/util/net/hostandport.h"
 
 namespace mongo {
@@ -55,8 +54,33 @@ class Collection;
 class Database;
 class RecordId;
 
+/**
+ * Used to commit work for LogOpForSharding. Used to keep track of changes in documents that are
+ * part of a chunk being migrated.
+ */
+class LogTransactionOperationsForShardingHandler final : public RecoveryUnit::Change {
+public:
+    /**
+     * Invariant: idObj should belong to a document that is part of the active chunk being migrated
+     */
+    LogTransactionOperationsForShardingHandler(ServiceContext* svcCtx,
+                                               const std::vector<repl::ReplOperation>& stmts,
+                                               const repl::OpTime& prepareOrCommitOpTime)
+        : _svcCtx(svcCtx), _stmts(stmts), _prepareOrCommitOpTime(prepareOrCommitOpTime) {}
+
+    void commit(boost::optional<Timestamp>) override;
+
+    void rollback() override{};
+
+private:
+    ServiceContext* _svcCtx;
+    std::vector<repl::ReplOperation> _stmts;
+    const repl::OpTime _prepareOrCommitOpTime;
+};
+
 class MigrationChunkClonerSourceLegacy final : public MigrationChunkClonerSource {
-    MONGO_DISALLOW_COPYING(MigrationChunkClonerSourceLegacy);
+    MigrationChunkClonerSourceLegacy(const MigrationChunkClonerSourceLegacy&) = delete;
+    MigrationChunkClonerSourceLegacy& operator=(const MigrationChunkClonerSourceLegacy&) = delete;
 
 public:
     MigrationChunkClonerSourceLegacy(MoveChunkRequest request,
@@ -81,7 +105,8 @@ public:
                     const repl::OpTime& opTime) override;
 
     void onUpdateOp(OperationContext* opCtx,
-                    const BSONObj& updatedDoc,
+                    boost::optional<BSONObj> preImageDoc,
+                    const BSONObj& postImageDoc,
                     const repl::OpTime& opTime,
                     const repl::OpTime& prePostImageOpTime) override;
 
@@ -120,7 +145,8 @@ public:
 
     /**
      * Called by the recipient shard. Populates the passed BSONArrayBuilder with a set of documents,
-     * which are part of the initial clone sequence.
+     * which are part of the initial clone sequence. Assumes that there is only one active caller
+     * to this method at a time (otherwise, it can cause corruption/crash).
      *
      * Returns OK status on success. If there were documents returned in the result argument, this
      * method should be called more times until the result is empty. If it returns failure, it is
@@ -176,8 +202,8 @@ public:
     std::shared_ptr<Notification<bool>> getNotificationForNextSessionMigrationBatch();
 
 private:
-    friend class DeleteNotificationStage;
     friend class LogOpForShardingHandler;
+    friend class LogTransactionOperationsForShardingHandler;
 
     // Represents the states in which the cloner can be
     enum State { kNew, kCloning, kDone };
@@ -192,7 +218,7 @@ private:
      * Synchronously invokes the recipient shard with the specified command and either returns the
      * command response (if succeeded) or the status, if the command failed.
      */
-    StatusWith<BSONObj> _callRecipient(const BSONObj& cmdObj);
+    StatusWith<BSONObj> _callRecipient(OperationContext* opCtx, const BSONObj& cmdObj);
 
     /**
      * Get the disklocs that belong to the chunk migrated and sort them in _cloneLocs (to avoid
@@ -202,6 +228,65 @@ private:
      */
     Status _storeCurrentLocs(OperationContext* opCtx);
 
+    /**
+     * Adds the OpTime to the list of OpTimes for oplog entries that we should consider migrating as
+     * part of session migration.
+     */
+    void _addToSessionMigrationOptimeQueue(
+        const repl::OpTime& opTime,
+        SessionCatalogMigrationSource::EntryAtOpTimeType entryAtOpTimeType);
+
+    void _addToSessionMigrationOptimeQueueForTransactionCommit(
+        const repl::OpTime& opTime,
+        SessionCatalogMigrationSource::EntryAtOpTimeType entryAtOpTimeType);
+
+    /*
+     * Appends the relevant document changes to the appropriate internal data structures (known
+     * colloquially as the 'transfer mods queue'). These structures track document changes that are
+     * part of a part of a chunk being migrated. In doing so, this the method also removes the
+     * corresponding operation track request from the operation track requests queue.
+     */
+    void _addToTransferModsQueue(const BSONObj& idObj,
+                                 const char op,
+                                 const repl::OpTime& opTime,
+                                 const repl::OpTime& prePostImageOpTime);
+
+    /**
+     * Adds an operation to the outstanding operation track requests. Returns false if the cloner
+     * is no longer accepting new operation track requests.
+     */
+    bool _addedOperationToOutstandingOperationTrackRequests();
+
+    /**
+     * Called to indicate a request to track an operation must be filled. The operations in
+     * question indicate a change to a document in the chunk being cloned. Increments a counter
+     * residing inside the MigrationChunkClonerSourceLegacy class.
+     *
+     * There should always be a one to one match from the number of calls to this function to the
+     * number of calls to the corresponding decrement* function.
+     *
+     * NOTE: This funtion invariants that we are currently accepting new operation track requests.
+     * It is up to callers of this function to make sure that will always be the case.
+     */
+    void _incrementOutstandingOperationTrackRequests(WithLock);
+
+    /**
+     * Called once a request to track an operation has been filled. The operations in question
+     * indicate a change to a document in the chunk being cloned. Decrements a counter residing
+     * inside the MigrationChunkClonerSourceLegacy class.
+     *
+     * There should always be a one to one match from the number of calls to this function to the
+     * number of calls to the corresponding increment* function.
+     */
+    void _decrementOutstandingOperationTrackRequests();
+
+    /**
+     * Waits for all outstanding operation track requests to be fulfilled before returning from this
+     * function. Should only be used in the cleanup for this class. Should use a lock wrapped
+     * around this class's mutex.
+     */
+    void _drainAllOutstandingOperationTrackRequests(stdx::unique_lock<Latch>& lk);
+
     /**
      * Appends to the builder the list of _id of documents that were deleted during migration.
      * Entries appended to the builder are removed from the list.
@@ -237,14 +322,10 @@ private:
     // The resolved primary of the recipient shard
     const HostAndPort _recipientHost;
 
-    // Registered deletion notifications plan executor, which will listen for document deletions
-    // during the cloning stage
-    std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> _deleteNotifyExec;
-
     std::unique_ptr<SessionCatalogMigrationSource> _sessionCatalogSource;
 
     // Protects the entries below
-    stdx::mutex _mutex;
+    Mutex _mutex = MONGO_MAKE_LATCH("MigrationChunkClonerSourceLegacy::_mutex");
 
     // The current state of the cloner
     State _state{kNew};
@@ -256,6 +337,16 @@ private:
     // pre-allocation (initial clone).
     uint64_t _averageObjectSizeForCloneLocs{0};
 
+    // Represents all of the requested but not yet fulfilled operations to be tracked, with regards
+    // to the chunk being cloned.
+    uint64_t _outstandingOperationTrackRequests{0};
+
+    // Signals to any waiters once all unresolved operation tracking requests have completed.
+    stdx::condition_variable _allOutstandingOperationTrackRequestsDrained;
+
+    // Indicates whether new requests to track an operation are accepted.
+    bool _acceptingNewOperationTrackRequests{true};
+
     // The estimated average object _id size during the clone phase.
     uint64_t _averageObjectIdSize{0};
 
