@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -38,6 +37,7 @@
 #include "mongo/db/s/session_catalog_migration_source.h"
 #include "mongo/db/session.h"
 #include "mongo/db/session_txn_record_gen.h"
+#include "mongo/db/transaction_participant.h"
 #include "mongo/executor/remote_command_request.h"
 #include "mongo/unittest/unittest.h"
 
@@ -47,6 +47,8 @@ namespace {
 using executor::RemoteCommandRequest;
 
 const NamespaceString kNs("a.b");
+const KeyPattern kShardKey(BSON("x" << 1));
+const ChunkRange kChunkRange(BSON("x" << 0), BSON("x" << 100));
 
 class SessionCatalogMigrationSourceTest : public MockReplCoordServerFixture {};
 
@@ -66,21 +68,21 @@ repl::OplogEntry makeOplogEntry(repl::OpTime opTime,
                                 boost::optional<OperationSessionInfo> osi,
                                 boost::optional<repl::RetryImageEnum> needsRetryImage) {
     return repl::OplogEntry(
-        opTime,                           // optime
-        0,                                // hash
-        opType,                           // opType
-        nss,                              // namespace
-        boost::none,                      // uuid
-        boost::none,                      // fromMigrate
-        repl::OplogEntry::kOplogVersion,  // version
-        object,                           // o
-        object2,                          // o2
-        osi.get_value_or({}),             // sessionInfo
-        boost::none,                      // upsert
-        wallClockTime,                    // wall clock time
-        stmtId,                           // statement id
-        prevWriteOpTimeInTransaction,     // optime of previous write within same transaction
-        preImageOpTime,                   // pre-image optime
+        opTime,                                // optime
+        0,                                     // hash
+        opType,                                // opType
+        nss,                                   // namespace
+        boost::none,                           // uuid
+        boost::none,                           // fromMigrate
+        repl::OplogEntry::kOplogVersion,       // version
+        object,                                // o
+        object2,                               // o2
+        osi.get_value_or({}),                  // sessionInfo
+        boost::none,                           // upsert
+        wallClockTime.get_value_or(Date_t()),  // wall clock time
+        stmtId,                                // statement id
+        prevWriteOpTimeInTransaction,          // optime of previous write within same transaction
+        preImageOpTime,                        // pre-image optime
         postImageOpTime,
         needsRetryImage);  // post-image optime
 }
@@ -102,7 +104,7 @@ repl::OplogEntry makeOplogEntry(
                           kNs,
                           object,
                           object2,
-                          wallClockTime,
+                          wallClockTime.get_value_or(Date_t()),
                           stmtId,
                           prevWriteOpTimeInTransaction,
                           preImageOpTime,
@@ -112,7 +114,7 @@ repl::OplogEntry makeOplogEntry(
 }
 
 TEST_F(SessionCatalogMigrationSourceTest, NoSessionsToTransferShouldNotHaveOplog) {
-    SessionCatalogMigrationSource migrationSource(opCtx(), kNs);
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
     ASSERT_FALSE(migrationSource.fetchNextOplog(opCtx()));
     ASSERT_FALSE(migrationSource.hasMoreOplog());
 }
@@ -131,7 +133,7 @@ TEST_F(SessionCatalogMigrationSourceTest, OneSessionWithTwoWrites) {
     auto entry2 =
         makeOplogEntry(repl::OpTime(Timestamp(67, 54801), 2),  // optime
                        repl::OpTypeEnum::kInsert,              // op type
-                       BSON("y" << 50),                        // o
+                       BSON("x" << 50),                        // o
                        boost::none,                            // o2
                        Date_t::now(),                          // wall clock time
                        1,                                      // statement id
@@ -147,7 +149,7 @@ TEST_F(SessionCatalogMigrationSourceTest, OneSessionWithTwoWrites) {
     DBDirectClient client(opCtx());
     client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), sessionRecord.toBSON());
 
-    SessionCatalogMigrationSource migrationSource(opCtx(), kNs);
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
     ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
 
     {
@@ -184,7 +186,7 @@ TEST_F(SessionCatalogMigrationSourceTest, TwoSessionWithTwoWrites) {
     auto entry1b =
         makeOplogEntry(repl::OpTime(Timestamp(67, 54801), 2),  // optime
                        repl::OpTypeEnum::kInsert,              // op type
-                       BSON("y" << 50),                        // o
+                       BSON("x" << 50),                        // o
                        boost::none,                            // o2
                        Date_t::now(),                          // wall clock time
                        1,                                      // statement id
@@ -212,7 +214,7 @@ TEST_F(SessionCatalogMigrationSourceTest, TwoSessionWithTwoWrites) {
     auto entry2b =
         makeOplogEntry(repl::OpTime(Timestamp(789, 13), 2),  // optime
                        repl::OpTypeEnum::kDelete,            // op type
-                       BSON("y" << 50),                      // o
+                       BSON("x" << 50),                      // o
                        boost::none,                          // o2
                        Date_t::now(),                        // wall clock time
                        4,                                    // statement id
@@ -232,7 +234,7 @@ TEST_F(SessionCatalogMigrationSourceTest, TwoSessionWithTwoWrites) {
     insertOplogEntry(entry1b);
     insertOplogEntry(entry2b);
 
-    SessionCatalogMigrationSource migrationSource(opCtx(), kNs);
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
     ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
 
     auto checkNextBatch = [this, &migrationSource](const repl::OplogEntry& firstExpectedOplog,
@@ -289,7 +291,7 @@ TEST_F(SessionCatalogMigrationSourceTest, OneSessionWithFindAndModifyPreImageAnd
     auto entry2 = makeOplogEntry(
         repl::OpTime(Timestamp(52, 346), 2),  // optime
         repl::OpTypeEnum::kDelete,            // op type
-        BSON("y" << 50),                      // o
+        BSON("x" << 50),                      // o
         boost::none,                          // o2
         Date_t::now(),                        // wall clock time
         1,                                    // statement id
@@ -310,8 +312,8 @@ TEST_F(SessionCatalogMigrationSourceTest, OneSessionWithFindAndModifyPreImageAnd
     auto entry4 =
         makeOplogEntry(repl::OpTime(Timestamp(73, 6), 2),  // optime
                        repl::OpTypeEnum::kUpdate,          // op type
-                       BSON("x" << 19),                    // o
-                       BSON("$inc" << BSON("x" << 1)),     // o2
+                       BSON("$inc" << BSON("x" << 1)),     // o
+                       BSON("x" << 19),                    // o2
                        Date_t::now(),                      // wall clock time
                        3,                                  // statement id
                        entry2.getOpTime(),   // optime of previous write within same transaction
@@ -328,7 +330,7 @@ TEST_F(SessionCatalogMigrationSourceTest, OneSessionWithFindAndModifyPreImageAnd
     DBDirectClient client(opCtx());
     client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), sessionRecord.toBSON());
 
-    SessionCatalogMigrationSource migrationSource(opCtx(), kNs);
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
     ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
 
     auto expectedSequece = {entry3, entry4, entry1, entry2};
@@ -347,7 +349,7 @@ TEST_F(SessionCatalogMigrationSourceTest, OneSessionWithFindAndModifyPreImageAnd
 
 TEST_F(SessionCatalogMigrationSourceTest, ForgeImageEntriesWhenFetchingEntriesWithNeedsRetryImage) {
     repl::ImageEntry imageEntry;
-    const auto preImage = BSON("_id" << 1 << "y" << 50);
+    const auto preImage = BSON("_id" << 1 << "x" << 50);
     const auto lsid = makeLogicalSessionIdForTest();
     const repl::OpTime imageEntryOpTime = repl::OpTime(Timestamp(52, 346), 2);
     const auto txnNumber = 1LL;
@@ -368,10 +370,10 @@ TEST_F(SessionCatalogMigrationSourceTest, ForgeImageEntriesWhenFetchingEntriesWi
     auto entry = makeOplogEntry(
         repl::OpTime(Timestamp(52, 346), 2),  // optime
         repl::OpTypeEnum::kDelete,            // op type
-        BSON("y" << 50),                      // o
+        BSON("x" << 50),                      // o
         boost::none,                          // o2
         Date_t::now(),                        // wall clock time
-        0,                                    // statement id
+        1,                                    // statement id
         repl::OpTime(Timestamp(0, 0), 0),     // optime of previous write within same transaction
         boost::none,                          // pre-image optime
         boost::none,                          // post-image optime
@@ -387,7 +389,7 @@ TEST_F(SessionCatalogMigrationSourceTest, ForgeImageEntriesWhenFetchingEntriesWi
 
     client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), sessionRecord.toBSON());
 
-    SessionCatalogMigrationSource migrationSource(opCtx(), kNs);
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
     // The next oplog entry should be the forged preImage entry.
     ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
     ASSERT_TRUE(migrationSource.hasMoreOplog());
@@ -399,7 +401,7 @@ TEST_F(SessionCatalogMigrationSourceTest, ForgeImageEntriesWhenFetchingEntriesWi
     ASSERT_EQUALS(txnNumber, nextOplogResult.oplog->getTxnNumber().get());
     ASSERT_EQUALS(lsid, nextOplogResult.oplog->getSessionId().get());
     ASSERT_EQUALS("n", repl::OpType_serializer(nextOplogResult.oplog->getOpType()));
-    ASSERT_EQUALS(0LL, nextOplogResult.oplog->getStatementId().get());
+    ASSERT_EQUALS(1LL, nextOplogResult.oplog->getStatementId().get());
 
     // The next oplog entry should be the original entry that generated the image entry.
     ASSERT_TRUE(migrationSource.hasMoreOplog());
@@ -454,7 +456,7 @@ TEST_F(SessionCatalogMigrationSourceTest, OplogWithOtherNsShouldBeIgnored) {
     client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(),
                   sessionRecord2.toBSON());
 
-    SessionCatalogMigrationSource migrationSource(opCtx(), kNs);
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
     ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
 
     ASSERT_TRUE(migrationSource.hasMoreOplog());
@@ -502,18 +504,20 @@ TEST_F(SessionCatalogMigrationSourceTest, SessionDumpWithMultipleNewWrites) {
     auto entry3 = makeOplogEntry(
         repl::OpTime(Timestamp(55, 12), 2),  // optime
         repl::OpTypeEnum::kInsert,           // op type
-        BSON("z" << 40),                     // o
+        BSON("x" << 40),                     // o
         boost::none,                         // o2
         Date_t::now(),                       // wall clock time
         2,                                   // statement id
         repl::OpTime(Timestamp(0, 0), 0));   // optime of previous write within same transaction
     insertOplogEntry(entry3);
 
-    SessionCatalogMigrationSource migrationSource(opCtx(), kNs);
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
     ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
 
-    migrationSource.notifyNewWriteOpTime(entry2.getOpTime());
-    migrationSource.notifyNewWriteOpTime(entry3.getOpTime());
+    migrationSource.notifyNewWriteOpTime(
+        entry2.getOpTime(), SessionCatalogMigrationSource::EntryAtOpTimeType::kRetryableWrite);
+    migrationSource.notifyNewWriteOpTime(
+        entry3.getOpTime(), SessionCatalogMigrationSource::EntryAtOpTimeType::kRetryableWrite);
 
     {
         ASSERT_TRUE(migrationSource.hasMoreOplog());
@@ -544,16 +548,18 @@ TEST_F(SessionCatalogMigrationSourceTest, SessionDumpWithMultipleNewWrites) {
 }
 
 TEST_F(SessionCatalogMigrationSourceTest, ShouldAssertIfOplogCannotBeFound) {
-    SessionCatalogMigrationSource migrationSource(opCtx(), kNs);
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
     ASSERT_FALSE(migrationSource.fetchNextOplog(opCtx()));
 
-    migrationSource.notifyNewWriteOpTime(repl::OpTime(Timestamp(100, 3), 1));
+    migrationSource.notifyNewWriteOpTime(
+        repl::OpTime(Timestamp(100, 3), 1),
+        SessionCatalogMigrationSource::EntryAtOpTimeType::kRetryableWrite);
     ASSERT_TRUE(migrationSource.hasMoreOplog());
     ASSERT_THROWS(migrationSource.fetchNextOplog(opCtx()), AssertionException);
 }
 
 TEST_F(SessionCatalogMigrationSourceTest, ShouldBeAbleInsertNewWritesAfterBufferWasDepleted) {
-    SessionCatalogMigrationSource migrationSource(opCtx(), kNs);
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
     ASSERT_FALSE(migrationSource.fetchNextOplog(opCtx()));
 
     {
@@ -567,7 +573,8 @@ TEST_F(SessionCatalogMigrationSourceTest, ShouldBeAbleInsertNewWritesAfterBuffer
             repl::OpTime(Timestamp(0, 0), 0));  // optime of previous write within same transaction
         insertOplogEntry(entry);
 
-        migrationSource.notifyNewWriteOpTime(entry.getOpTime());
+        migrationSource.notifyNewWriteOpTime(
+            entry.getOpTime(), SessionCatalogMigrationSource::EntryAtOpTimeType::kRetryableWrite);
 
         ASSERT_TRUE(migrationSource.hasMoreOplog());
         ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
@@ -591,7 +598,8 @@ TEST_F(SessionCatalogMigrationSourceTest, ShouldBeAbleInsertNewWritesAfterBuffer
             repl::OpTime(Timestamp(0, 0), 0));   // optime of previous write within same transaction
         insertOplogEntry(entry);
 
-        migrationSource.notifyNewWriteOpTime(entry.getOpTime());
+        migrationSource.notifyNewWriteOpTime(
+            entry.getOpTime(), SessionCatalogMigrationSource::EntryAtOpTimeType::kRetryableWrite);
 
         ASSERT_TRUE(migrationSource.hasMoreOplog());
         ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
@@ -607,14 +615,15 @@ TEST_F(SessionCatalogMigrationSourceTest, ShouldBeAbleInsertNewWritesAfterBuffer
         auto entry = makeOplogEntry(
             repl::OpTime(Timestamp(55, 12), 2),  // optime
             repl::OpTypeEnum::kInsert,           // op type
-            BSON("z" << 40),                     // o
+            BSON("x" << 40),                     // o
             boost::none,                         // o2
             Date_t::now(),                       // wall clock time
             2,                                   // statement id
             repl::OpTime(Timestamp(0, 0), 0));   // optime of previous write within same transaction
         insertOplogEntry(entry);
 
-        migrationSource.notifyNewWriteOpTime(entry.getOpTime());
+        migrationSource.notifyNewWriteOpTime(
+            entry.getOpTime(), SessionCatalogMigrationSource::EntryAtOpTimeType::kRetryableWrite);
 
         ASSERT_TRUE(migrationSource.hasMoreOplog());
         ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
@@ -649,7 +658,7 @@ TEST_F(SessionCatalogMigrationSourceTest, ReturnsDeadEndSentinelForIncompleteHis
     DBDirectClient client(opCtx());
     client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), sessionRecord.toBSON());
 
-    SessionCatalogMigrationSource migrationSource(opCtx(), kNs);
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
     ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
 
     {
@@ -668,7 +677,7 @@ TEST_F(SessionCatalogMigrationSourceTest, ReturnsDeadEndSentinelForIncompleteHis
 
         auto oplog = *nextOplogResult.oplog;
         ASSERT_TRUE(oplog.getObject2());
-        ASSERT_BSONOBJ_EQ(Session::kDeadEndSentinel, *oplog.getObject2());
+        ASSERT_BSONOBJ_EQ(TransactionParticipant::kDeadEndSentinel, *oplog.getObject2());
         ASSERT_TRUE(oplog.getStatementId());
         ASSERT_EQ(kIncompleteHistoryStmtId, *oplog.getStatementId());
         ASSERT_TRUE(oplog.getWallClockTime());
@@ -707,7 +716,7 @@ TEST_F(SessionCatalogMigrationSourceTest, ShouldAssertWhenRollbackDetected) {
     DBDirectClient client(opCtx());
     client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), sessionRecord.toBSON());
 
-    SessionCatalogMigrationSource migrationSource(opCtx(), kNs);
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
     ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
 
     {
@@ -724,5 +733,445 @@ TEST_F(SessionCatalogMigrationSourceTest, ShouldAssertWhenRollbackDetected) {
     ASSERT_TRUE(migrationSource.hasMoreOplog());
 }
 
+TEST_F(SessionCatalogMigrationSourceTest,
+       CommitTransactionEntriesShouldBeConvertedToDeadEndSentinel) {
+    SessionTxnRecord txnRecord;
+    txnRecord.setSessionId(makeLogicalSessionIdForTest());
+    txnRecord.setTxnNum(20);
+    txnRecord.setLastWriteOpTime(repl::OpTime(Timestamp(12, 34), 5));
+    txnRecord.setLastWriteDate(Date_t::now());
+    txnRecord.setState(DurableTxnStateEnum::kCommitted);
+
+    DBDirectClient client(opCtx());
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), txnRecord.toBSON());
+
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
+
+    ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
+    ASSERT_TRUE(migrationSource.hasMoreOplog());
+
+    auto nextOplogResult = migrationSource.getLastFetchedOplog();
+    ASSERT_FALSE(nextOplogResult.shouldWaitForMajority);
+    // Cannot compare directly because of SERVER-31356
+    ASSERT_BSONOBJ_EQ(TransactionParticipant::kDeadEndSentinel,
+                      *nextOplogResult.oplog->getObject2());
+
+    ASSERT_FALSE(migrationSource.fetchNextOplog(opCtx()));
+}
+
+TEST_F(SessionCatalogMigrationSourceTest,
+       PrepareTransactionEntriesShouldBeConvertedToDeadEndSentinel) {
+    SessionTxnRecord txnRecord;
+    txnRecord.setSessionId(makeLogicalSessionIdForTest());
+    txnRecord.setTxnNum(20);
+    txnRecord.setLastWriteOpTime(repl::OpTime(Timestamp(12, 34), 5));
+    txnRecord.setLastWriteDate(Date_t::now());
+    txnRecord.setState(DurableTxnStateEnum::kPrepared);
+
+    DBDirectClient client(opCtx());
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), txnRecord.toBSON());
+
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
+
+    ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
+    ASSERT_TRUE(migrationSource.hasMoreOplog());
+
+    auto nextOplogResult = migrationSource.getLastFetchedOplog();
+    ASSERT_FALSE(nextOplogResult.shouldWaitForMajority);
+    // Cannot compare directly because of SERVER-31356
+    ASSERT_BSONOBJ_EQ(TransactionParticipant::kDeadEndSentinel,
+                      *nextOplogResult.oplog->getObject2());
+
+    ASSERT_FALSE(migrationSource.fetchNextOplog(opCtx()));
+}
+
+TEST_F(SessionCatalogMigrationSourceTest, InProgressTransactionEntriesShouldBeIgnored) {
+    SessionTxnRecord txnRecord;
+    txnRecord.setSessionId(makeLogicalSessionIdForTest());
+    txnRecord.setTxnNum(20);
+    txnRecord.setLastWriteOpTime(repl::OpTime(Timestamp(12, 34), 5));
+    txnRecord.setLastWriteDate(Date_t::now());
+    txnRecord.setState(DurableTxnStateEnum::kInProgress);
+
+    DBDirectClient client(opCtx());
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), txnRecord.toBSON());
+
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
+
+    ASSERT_FALSE(migrationSource.fetchNextOplog(opCtx()));
+    ASSERT_FALSE(migrationSource.hasMoreOplog());
+}
+
+TEST_F(SessionCatalogMigrationSourceTest, AbortedTransactionEntriesShouldBeIgnored) {
+    SessionTxnRecord txnRecord;
+    txnRecord.setSessionId(makeLogicalSessionIdForTest());
+    txnRecord.setTxnNum(20);
+    txnRecord.setLastWriteOpTime(repl::OpTime(Timestamp(12, 34), 5));
+    txnRecord.setLastWriteDate(Date_t::now());
+    txnRecord.setState(DurableTxnStateEnum::kAborted);
+
+    DBDirectClient client(opCtx());
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), txnRecord.toBSON());
+
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
+
+    ASSERT_FALSE(migrationSource.fetchNextOplog(opCtx()));
+    ASSERT_FALSE(migrationSource.hasMoreOplog());
+}
+
+TEST_F(SessionCatalogMigrationSourceTest,
+       MixedTransactionEntriesAndRetryableWritesEntriesReturnCorrectResults) {
+    // Create an entry corresponding to a retryable write.
+    auto insertOplog = makeOplogEntry(
+        repl::OpTime(Timestamp(52, 345), 2),  // optime
+        repl::OpTypeEnum::kInsert,            // op type
+        BSON("x" << 30),                      // o
+        boost::none,                          // o2
+        Date_t::now(),                        // wall clock time
+        0,                                    // statement id
+        repl::OpTime(Timestamp(0, 0), 0));    // optime of previous write within same transaction
+
+    // Create a config.transaction entry pointing to the insert oplog entry.
+    SessionTxnRecord retryableWriteRecord;
+    retryableWriteRecord.setSessionId(makeLogicalSessionIdForTest());
+    retryableWriteRecord.setTxnNum(1);
+    retryableWriteRecord.setLastWriteOpTime(insertOplog.getOpTime());
+    retryableWriteRecord.setLastWriteDate(*insertOplog.getWallClockTime());
+
+    // Create a config.transaction entry pointing to an imaginary commitTransaction entry.
+    SessionTxnRecord txnRecord;
+    txnRecord.setSessionId(makeLogicalSessionIdForTest());
+    txnRecord.setTxnNum(1);
+    txnRecord.setLastWriteOpTime(repl::OpTime(Timestamp(12, 34), 2));
+    txnRecord.setLastWriteDate(Date_t::now());
+    txnRecord.setState(DurableTxnStateEnum::kCommitted);
+
+    // Insert both entries into the config.transactions table.
+    DBDirectClient client(opCtx());
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(),
+                  retryableWriteRecord.toBSON());
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), txnRecord.toBSON());
+
+    // Insert the 'insert' oplog entry into the oplog.
+    insertOplogEntry(insertOplog);
+
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
+
+    // Function to verify the oplog entry corresponding to the retryable write.
+    auto checkRetryableWriteEntry = [&] {
+        ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
+        ASSERT_TRUE(migrationSource.hasMoreOplog());
+
+        auto nextOplogResult = migrationSource.getLastFetchedOplog();
+        ASSERT_FALSE(nextOplogResult.shouldWaitForMajority);
+        // Cannot compare directly because of SERVER-31356
+        ASSERT_BSONOBJ_EQ(insertOplog.toBSON(), nextOplogResult.oplog->toBSON());
+    };
+
+    // Function to verify the oplog entry corresponding to the transaction.
+    auto checkTxnEntry = [&] {
+        ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
+        ASSERT_TRUE(migrationSource.hasMoreOplog());
+
+        auto nextOplogResult = migrationSource.getLastFetchedOplog();
+        ASSERT_FALSE(nextOplogResult.shouldWaitForMajority);
+        // Cannot compare directly because of SERVER-31356
+        ASSERT_BSONOBJ_EQ(TransactionParticipant::kDeadEndSentinel,
+                          *nextOplogResult.oplog->getObject2());
+    };
+
+    // Logical session ids are generated randomly and the migration source queries in order of
+    // logical session id, so we need to change the order of the checks depending on the ordering of
+    // the lsids between the retryable write record and the transaction record.
+    if (retryableWriteRecord.getSessionId().toBSON().woCompare(txnRecord.getSessionId().toBSON()) <
+        0) {
+        checkTxnEntry();
+        checkRetryableWriteEntry();
+    } else {
+        checkRetryableWriteEntry();
+        checkTxnEntry();
+    }
+
+    ASSERT_FALSE(migrationSource.fetchNextOplog(opCtx()));
+}
+
+TEST_F(SessionCatalogMigrationSourceTest, FindAndModifyDeleteNotTouchingChunkIsIgnored) {
+    auto entry1 = makeOplogEntry(
+        repl::OpTime(Timestamp(52, 345), 2),  // optime
+        repl::OpTypeEnum::kNoop,              // op type
+        BSON("x" << -50),                     // o
+        boost::none,                          // o2
+        Date_t::now(),                        // wall clock time
+        0,                                    // statement id
+        repl::OpTime(Timestamp(0, 0), 0));    // optime of previous write within same transaction
+    insertOplogEntry(entry1);
+
+    auto entry2 = makeOplogEntry(
+        repl::OpTime(Timestamp(52, 346), 2),  // optime
+        repl::OpTypeEnum::kDelete,            // op type
+        BSON("x" << -50),                     // o
+        boost::none,                          // o2
+        Date_t::now(),                        // wall clock time
+        1,                                    // statement id
+        repl::OpTime(Timestamp(0, 0), 0),     // optime of previous write within same transaction
+        entry1.getOpTime());                  // pre-image optime
+    insertOplogEntry(entry2);
+
+    SessionTxnRecord sessionRecord;
+    sessionRecord.setSessionId(makeLogicalSessionIdForTest());
+    sessionRecord.setTxnNum(1);
+    sessionRecord.setLastWriteOpTime(entry2.getOpTime());
+    sessionRecord.setLastWriteDate(*entry2.getWallClockTime());
+
+    DBDirectClient client(opCtx());
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), sessionRecord.toBSON());
+
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
+    ASSERT_FALSE(migrationSource.fetchNextOplog(opCtx()));
+}
+
+TEST_F(SessionCatalogMigrationSourceTest, FindAndModifyUpdatePrePostNotTouchingChunkIsIgnored) {
+    auto entry1 = makeOplogEntry(
+        repl::OpTime(Timestamp(52, 345), 2),  // optime
+        repl::OpTypeEnum::kNoop,              // op type
+        BSON("x" << -5),                      // o
+        boost::none,                          // o2
+        Date_t::now(),                        // wall clock time
+        0,                                    // statement id
+        repl::OpTime(Timestamp(0, 0), 0));    // optime of previous write within same transaction
+    insertOplogEntry(entry1);
+
+    auto entry2 = makeOplogEntry(
+        repl::OpTime(Timestamp(52, 346), 2),  // optime
+        repl::OpTypeEnum::kUpdate,            // op type
+        BSON("$set" << BSON("y" << 1)),       // o
+        BSON("x" << -5),                      // o2
+        Date_t::now(),                        // wall clock time
+        1,                                    // statement id
+        repl::OpTime(Timestamp(0, 0), 0),     // optime of previous write within same transaction
+        entry1.getOpTime());                  // pre-image optime
+    insertOplogEntry(entry2);
+
+    SessionTxnRecord sessionRecord;
+    sessionRecord.setSessionId(makeLogicalSessionIdForTest());
+    sessionRecord.setTxnNum(1);
+    sessionRecord.setLastWriteOpTime(entry2.getOpTime());
+    sessionRecord.setLastWriteDate(*entry2.getWallClockTime());
+
+    DBDirectClient client(opCtx());
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), sessionRecord.toBSON());
+
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
+    ASSERT_FALSE(migrationSource.fetchNextOplog(opCtx()));
+}
+
+TEST_F(SessionCatalogMigrationSourceTest,
+       UpdatePreImageTouchingPostNotTouchingChunkShouldNotBeIgnored) {
+    auto entry1 = makeOplogEntry(
+        repl::OpTime(Timestamp(52, 345), 2),  // optime
+        repl::OpTypeEnum::kNoop,              // op type
+        BSON("x" << -50),                     // o
+        boost::none,                          // o2
+        Date_t::now(),                        // wall clock time
+        0,                                    // statement id
+        repl::OpTime(Timestamp(0, 0), 0));    // optime of previous write within same transaction
+    insertOplogEntry(entry1);
+
+    auto entry2 = makeOplogEntry(
+        repl::OpTime(Timestamp(52, 346), 2),  // optime
+        repl::OpTypeEnum::kUpdate,            // op type
+        BSON("$set" << BSON("x" << -50)),     // o
+        BSON("x" << 10),                      // o2
+        Date_t::now(),                        // wall clock time
+        1,                                    // statement id
+        repl::OpTime(Timestamp(0, 0), 0),     // optime of previous write within same transaction
+        boost::none,                          // pre-image optime
+        entry1.getOpTime());                  // post-image optime
+    insertOplogEntry(entry2);
+
+    SessionTxnRecord sessionRecord;
+    sessionRecord.setSessionId(makeLogicalSessionIdForTest());
+    sessionRecord.setTxnNum(1);
+    sessionRecord.setLastWriteOpTime(entry2.getOpTime());
+    sessionRecord.setLastWriteDate(*entry2.getWallClockTime());
+
+    DBDirectClient client(opCtx());
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), sessionRecord.toBSON());
+
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
+    ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
+
+    auto expectedSequece = {entry1, entry2};
+
+    for (auto oplog : expectedSequece) {
+        ASSERT_TRUE(migrationSource.hasMoreOplog());
+        auto nextOplogResult = migrationSource.getLastFetchedOplog();
+        ASSERT_FALSE(nextOplogResult.shouldWaitForMajority);
+        // Cannot compare directly because of SERVER-31356
+        ASSERT_BSONOBJ_EQ(oplog.toBSON(), nextOplogResult.oplog->toBSON());
+        migrationSource.fetchNextOplog(opCtx());
+    }
+
+    ASSERT_FALSE(migrationSource.hasMoreOplog());
+}
+
+TEST_F(SessionCatalogMigrationSourceTest,
+       UpdatePreImageNotTouchingPostTouchingChunkShouldBeIgnored) {
+    auto entry1 = makeOplogEntry(
+        repl::OpTime(Timestamp(52, 345), 2),  // optime
+        repl::OpTypeEnum::kNoop,              // op type
+        BSON("x" << 50),                      // o
+        boost::none,                          // o2
+        Date_t::now(),                        // wall clock time
+        0,                                    // statement id
+        repl::OpTime(Timestamp(0, 0), 0));    // optime of previous write within same transaction
+    insertOplogEntry(entry1);
+
+    auto entry2 = makeOplogEntry(
+        repl::OpTime(Timestamp(52, 346), 2),  // optime
+        repl::OpTypeEnum::kUpdate,            // op type
+        BSON("$set" << BSON("x" << 50)),      // o
+        BSON("x" << -10),                     // o2
+        Date_t::now(),                        // wall clock time
+        1,                                    // statement id
+        repl::OpTime(Timestamp(0, 0), 0),     // optime of previous write within same transaction
+        boost::none,                          // pre-image optime
+        entry1.getOpTime());                  // post-image optime
+    insertOplogEntry(entry2);
+
+    SessionTxnRecord sessionRecord;
+    sessionRecord.setSessionId(makeLogicalSessionIdForTest());
+    sessionRecord.setTxnNum(1);
+    sessionRecord.setLastWriteOpTime(entry2.getOpTime());
+    sessionRecord.setLastWriteDate(*entry2.getWallClockTime());
+
+    DBDirectClient client(opCtx());
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), sessionRecord.toBSON());
+
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
+    ASSERT_FALSE(migrationSource.fetchNextOplog(opCtx()));
+}
+
+TEST_F(SessionCatalogMigrationSourceTest, FindAndModifyUpdateNotTouchingChunkShouldBeIgnored) {
+    auto entry1 = makeOplogEntry(
+        repl::OpTime(Timestamp(52, 345), 2),  // optime
+        repl::OpTypeEnum::kNoop,              // op type
+        BSON("x" << -10 << "y" << 50),        // o
+        boost::none,                          // o2
+        Date_t::now(),                        // wall clock time
+        0,                                    // statement id
+        repl::OpTime(Timestamp(0, 0), 0));    // optime of previous write within same transaction
+    insertOplogEntry(entry1);
+
+    auto entry2 = makeOplogEntry(
+        repl::OpTime(Timestamp(52, 346), 2),  // optime
+        repl::OpTypeEnum::kUpdate,            // op type
+        BSON("$set" << BSON("y" << 50)),      // o
+        BSON("x" << -10),                     // o2
+        Date_t::now(),                        // wall clock time
+        1,                                    // statement id
+        repl::OpTime(Timestamp(0, 0), 0),     // optime of previous write within same transaction
+        boost::none,                          // pre-image optime
+        entry1.getOpTime());                  // post-image optime
+    insertOplogEntry(entry2);
+
+    SessionTxnRecord sessionRecord;
+    sessionRecord.setSessionId(makeLogicalSessionIdForTest());
+    sessionRecord.setTxnNum(1);
+    sessionRecord.setLastWriteOpTime(entry2.getOpTime());
+    sessionRecord.setLastWriteDate(*entry2.getWallClockTime());
+
+    DBDirectClient client(opCtx());
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(), sessionRecord.toBSON());
+
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
+    ASSERT_FALSE(migrationSource.fetchNextOplog(opCtx()));
+}
+
+TEST_F(SessionCatalogMigrationSourceTest, TwoSessionWithTwoWritesContainingWriteNotInChunk) {
+    auto sessionId1 = makeLogicalSessionIdForTest();
+    auto sessionId2 = makeLogicalSessionIdForTest();
+
+    auto cmpResult = sessionId1.toBSON().woCompare(sessionId2.toBSON());
+    auto lowerSessionId = (cmpResult < 0) ? sessionId1 : sessionId2;
+    auto higherSessionId = (cmpResult < 0) ? sessionId2 : sessionId1;
+
+    auto entry1a = makeOplogEntry(
+        repl::OpTime(Timestamp(52, 345), 2),  // optime
+        repl::OpTypeEnum::kInsert,            // op type
+        BSON("x" << 30),                      // o
+        boost::none,                          // o2
+        Date_t::now(),                        // wall clock time
+        0,                                    // statement id
+        repl::OpTime(Timestamp(0, 0), 0));    // optime of previous write within same transaction
+
+    auto entry1b =
+        makeOplogEntry(repl::OpTime(Timestamp(67, 54801), 2),  // optime
+                       repl::OpTypeEnum::kInsert,              // op type
+                       BSON("x" << -50),                       // o
+                       boost::none,                            // o2
+                       Date_t::now(),                          // wall clock time
+                       1,                                      // statement id
+                       entry1a.getOpTime());  // optime of previous write within same transaction
+
+    SessionTxnRecord sessionRecord1;
+    sessionRecord1.setSessionId(higherSessionId);
+    sessionRecord1.setTxnNum(1);
+    sessionRecord1.setLastWriteOpTime(entry1b.getOpTime());
+    sessionRecord1.setLastWriteDate(*entry1b.getWallClockTime());
+
+    DBDirectClient client(opCtx());
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(),
+                  sessionRecord1.toBSON());
+
+    auto entry2a = makeOplogEntry(
+        repl::OpTime(Timestamp(43, 12), 2),  // optime
+        repl::OpTypeEnum::kDelete,           // op type
+        BSON("x" << 30),                     // o
+        boost::none,                         // o2
+        Date_t::now(),                       // wall clock time
+        3,                                   // statement id
+        repl::OpTime(Timestamp(0, 0), 0));   // optime of previous write within same transaction
+
+    auto entry2b =
+        makeOplogEntry(repl::OpTime(Timestamp(789, 13), 2),  // optime
+                       repl::OpTypeEnum::kDelete,            // op type
+                       BSON("x" << 50),                      // o
+                       boost::none,                          // o2
+                       Date_t::now(),                        // wall clock time
+                       4,                                    // statement id
+                       entry2a.getOpTime());  // optime of previous write within same transaction
+
+    SessionTxnRecord sessionRecord2;
+    sessionRecord2.setSessionId(lowerSessionId);
+    sessionRecord2.setTxnNum(1);
+    sessionRecord2.setLastWriteOpTime(entry2b.getOpTime());
+    sessionRecord2.setLastWriteDate(*entry2b.getWallClockTime());
+
+    client.insert(NamespaceString::kSessionTransactionsTableNamespace.ns(),
+                  sessionRecord2.toBSON());
+
+    insertOplogEntry(entry2a);
+    insertOplogEntry(entry1a);
+    insertOplogEntry(entry1b);
+    insertOplogEntry(entry2b);
+
+    SessionCatalogMigrationSource migrationSource(opCtx(), kNs, kChunkRange, kShardKey);
+    ASSERT_TRUE(migrationSource.fetchNextOplog(opCtx()));
+
+    auto expectedSequece = {entry1a, entry2b, entry2a};
+
+    for (auto oplog : expectedSequece) {
+        ASSERT_TRUE(migrationSource.hasMoreOplog());
+        auto nextOplogResult = migrationSource.getLastFetchedOplog();
+        ASSERT_FALSE(nextOplogResult.shouldWaitForMajority);
+        // Cannot compare directly because of SERVER-31356
+        ASSERT_BSONOBJ_EQ(oplog.toBSON(), nextOplogResult.oplog->toBSON());
+        migrationSource.fetchNextOplog(opCtx());
+    }
+
+    ASSERT_FALSE(migrationSource.hasMoreOplog());
+}
+
 }  // namespace
 }  // namespace mongo
