@@ -90,14 +90,14 @@ bool checkIfSingleDoc(OperationContext* opCtx,
 
 /**
  * Checks the collection's metadata for a successful split on the specified chunkRange using the
- * specified splitKeys. Returns false if the metadata's chunks don't match the new chunk
+ * specified split points. Returns false if the metadata's chunks don't match the new chunk
  * boundaries exactly.
  */
 bool checkMetadataForSuccessfulSplitChunk(OperationContext* opCtx,
                                           const NamespaceString& nss,
                                           const OID& epoch,
                                           const ChunkRange& chunkRange,
-                                          const std::vector<BSONObj>& splitKeys) {
+                                          const std::vector<BSONObj>& splitPoints) {
     const auto metadataAfterSplit = [&] {
         AutoGetCollection autoColl(opCtx, nss, MODE_IS);
         return CollectionShardingState::get(opCtx, nss)->getCurrentMetadata();
@@ -107,19 +107,21 @@ bool checkMetadataForSuccessfulSplitChunk(OperationContext* opCtx,
             str::stream() << "Collection " << nss.ns() << " changed since split start",
             metadataAfterSplit->getCollVersion().epoch() == epoch);
 
-    auto newChunkBounds(splitKeys);
-    auto startKey = chunkRange.getMin();
-    newChunkBounds.push_back(chunkRange.getMax());
-
     ChunkType nextChunk;
-    for (const auto& endKey : newChunkBounds) {
+    for (auto it = splitPoints.begin(); it != splitPoints.end(); ++it) {
         // Check that all new chunks fit the new chunk boundaries
-        if (!metadataAfterSplit->getNextChunk(startKey, &nextChunk) ||
-            nextChunk.getMax().woCompare(endKey)) {
+        const auto& currentChunkMinKey =
+            it == splitPoints.begin() ? chunkRange.getMin() : *std::prev(it);
+        const auto& currentChunkMaxKey = *it;
+        if (!metadataAfterSplit->getNextChunk(currentChunkMinKey, &nextChunk) ||
+            nextChunk.getMax().woCompare(currentChunkMaxKey)) {
             return false;
         }
-
-        startKey = endKey;
+    }
+    // Special check for the last chunk produced.
+    if (!metadataAfterSplit->getNextChunk(splitPoints.back(), &nextChunk) ||
+        nextChunk.getMax().woCompare(chunkRange.getMax())) {
+        return false;
     }
 
     return true;
@@ -131,7 +133,7 @@ StatusWith<boost::optional<ChunkRange>> splitChunk(OperationContext* opCtx,
                                                    const NamespaceString& nss,
                                                    const BSONObj& keyPatternObj,
                                                    const ChunkRange& chunkRange,
-                                                   const std::vector<BSONObj>& splitKeys,
+                                                   std::vector<BSONObj>&& splitPoints,
                                                    const std::string& shardName,
                                                    const OID& expectedCollectionEpoch) {
     auto scopedSplitOrMergeChunk(uassertStatusOK(
@@ -140,15 +142,15 @@ StatusWith<boost::optional<ChunkRange>> splitChunk(OperationContext* opCtx,
     // If the shard key is hashed, then we must make sure that the split points are of type
     // NumberLong.
     if (KeyPattern::isHashedKeyPattern(keyPatternObj)) {
-        for (BSONObj splitKey : splitKeys) {
-            BSONObjIterator it(splitKey);
+        for (BSONObj splitPoint : splitPoints) {
+            BSONObjIterator it(splitPoint);
             while (it.more()) {
-                BSONElement splitKeyElement = it.next();
-                if (splitKeyElement.type() != NumberLong) {
+                BSONElement splitPointElement = it.next();
+                if (splitPointElement.type() != NumberLong) {
                     return {ErrorCodes::CannotSplit,
                             str::stream()
                                 << "splitChunk cannot split chunk " << chunkRange.toString()
-                                << ", split point " << splitKeyElement.toString()
+                                << ", split point " << splitPointElement.toString()
                                 << " must be of type "
                                    "NumberLong for hashed shard key patterns"};
                 }
@@ -157,8 +159,8 @@ StatusWith<boost::optional<ChunkRange>> splitChunk(OperationContext* opCtx,
     }
 
     // Commit the split to the config server.
-    auto request =
-        SplitChunkRequest(nss, shardName, expectedCollectionEpoch, chunkRange, splitKeys);
+    auto request = SplitChunkRequest(
+        nss, shardName, expectedCollectionEpoch, chunkRange, std::move(splitPoints));
 
     auto configCmdObj =
         request.toConfigCommandBSON(ShardingCatalogClient::kMajorityWriteConcern.toBSON());
@@ -207,7 +209,7 @@ StatusWith<boost::optional<ChunkRange>> splitChunk(OperationContext* opCtx,
     //
     if (!commandStatus.isOK() || !writeConcernStatus.isOK()) {
         if (checkMetadataForSuccessfulSplitChunk(
-                opCtx, nss, expectedCollectionEpoch, chunkRange, splitKeys)) {
+                opCtx, nss, expectedCollectionEpoch, chunkRange, request.getSplitPoints())) {
             // Split was committed.
         } else if (!commandStatus.isOK()) {
             return commandStatus;
@@ -234,12 +236,12 @@ StatusWith<boost::optional<ChunkRange>> splitChunk(OperationContext* opCtx,
     }
 
     auto backChunk = ChunkType();
-    backChunk.setMin(splitKeys.back());
+    backChunk.setMin(request.getSplitPoints().back());
     backChunk.setMax(chunkRange.getMax());
 
     auto frontChunk = ChunkType();
     frontChunk.setMin(chunkRange.getMin());
-    frontChunk.setMax(splitKeys.front());
+    frontChunk.setMax(request.getSplitPoints().front());
 
     KeyPattern shardKeyPattern(keyPatternObj);
     if (shardKeyPattern.globalMax().woCompare(backChunk.getMax()) == 0 &&
