@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -31,24 +30,25 @@
 #pragma once
 
 #include <boost/optional.hpp>
+#include <functional>
 #include <vector>
 
-#include "mongo/base/disallow_copying.h"
 #include "mongo/base/global_initializer_registerer.h"
 #include "mongo/db/logical_session_id.h"
 #include "mongo/db/operation_id.h"
 #include "mongo/db/storage/storage_engine.h"
 #include "mongo/platform/atomic_word.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
-#include "mongo/stdx/functional.h"
 #include "mongo/stdx/list.h"
 #include "mongo/stdx/memory.h"
-#include "mongo/stdx/mutex.h"
 #include "mongo/stdx/unordered_set.h"
 #include "mongo/transport/service_executor.h"
 #include "mongo/transport/session.h"
 #include "mongo/util/clock_source.h"
+#include "mongo/util/concurrency/with_lock.h"
 #include "mongo/util/decorable.h"
+#include "mongo/util/hierarchical_acquisition.h"
 #include "mongo/util/periodic_runner.h"
 #include "mongo/util/tick_source.h"
 
@@ -71,7 +71,8 @@ class TransportLayer;
  * including limitations on the lifetime of registered listeners.
  */
 class KillOpListenerInterface {
-    MONGO_DISALLOW_COPYING(KillOpListenerInterface);
+    KillOpListenerInterface(const KillOpListenerInterface&) = delete;
+    KillOpListenerInterface& operator=(const KillOpListenerInterface&) = delete;
 
 public:
     /**
@@ -96,7 +97,8 @@ protected:
  * zero or more Clients, which in turn each own OperationContexts.
  */
 class ServiceContext final : public Decorable<ServiceContext> {
-    MONGO_DISALLOW_COPYING(ServiceContext);
+    ServiceContext(const ServiceContext&) = delete;
+    ServiceContext& operator=(const ServiceContext&) = delete;
 
 public:
     /**
@@ -163,7 +165,7 @@ public:
         Client* next();
 
     private:
-        stdx::unique_lock<stdx::mutex> _lock;
+        stdx::unique_lock<Latch> _lock;
         ClientSet::const_iterator _curr;
         ClientSet::const_iterator _end;
     };
@@ -211,14 +213,14 @@ public:
      * Register a function of this type using  an instance of ConstructorActionRegisterer,
      * below, to cause the function to be executed on new ServiceContext instances.
      */
-    using ConstructorAction = stdx::function<void(ServiceContext*)>;
+    using ConstructorAction = std::function<void(ServiceContext*)>;
 
     /**
      * Register a function of this type using an instance of ConstructorActionRegisterer,
      * below, to cause the function to be executed on ServiceContext instances before they
      * are destroyed.
      */
-    using DestructorAction = stdx::function<void(ServiceContext*) noexcept>;
+    using DestructorAction = std::function<void(ServiceContext*)>;
 
     /**
      * Representation of a paired ConstructorAction and DestructorAction.
@@ -270,6 +272,23 @@ public:
                                     ConstructorAction constructor,
                                     DestructorAction destructor = {});
 
+        /**
+         * This constructor registers a constructor and optional destructor with the given
+         * "name", a list of names of prerequisites, "prereqs", and a list of names of dependents,
+         * "dependents".
+         *
+         * The named constructor will run after all of its prereqs successfully complete,
+         * and the corresponding destructor, if provided, will run before any of its
+         * prerequisites execute. The dependents will run after this constructor and
+         * the corresponding destructor, if provided, will run after any of its
+         * dependents execute.
+         */
+        ConstructorActionRegisterer(std::string name,
+                                    std::vector<std::string> prereqs,
+                                    std::vector<std::string> dependents,
+                                    ConstructorAction constructor,
+                                    DestructorAction destructor = {});
+
     private:
         using ConstructorActionListIterator = stdx::list<ConstructorDestructorActions>::iterator;
         ConstructorActionListIterator _iter;
@@ -356,18 +375,12 @@ public:
     /**
      * Kills the operation "opCtx" with the code "killCode", if opCtx has not already been killed.
      * Caller must own the lock on opCtx->getClient, and opCtx->getServiceContext() must be the same
-     *as
-     * this service context.
+     * as this service context. WithLock expects that the client lock be passed in.
      **/
-    void killOperation(OperationContext* opCtx,
+    void killOperation(WithLock,
+                       OperationContext* opCtx,
                        ErrorCodes::Error killCode = ErrorCodes::Interrupted);
 
-    /**
-     * Kills all operations that have a Client that is associated with an incoming user
-     * connection, except for the one associated with opCtx.
-     */
-    void killAllUserOperations(const OperationContext* opCtx, ErrorCodes::Error killCode);
-
     /**
      * Registers a listener to be notified each time an op is killed.
      *
@@ -509,6 +522,11 @@ public:
      */
     void setServiceExecutor(std::unique_ptr<transport::ServiceExecutor> exec);
 
+    /**
+     * Creates a delayed execution baton with basic functionality
+     */
+    BatonHandle makeBaton(OperationContext* opCtx) const;
+
 private:
     class ClientObserverHolder {
     public:
@@ -531,7 +549,7 @@ private:
         std::unique_ptr<ClientObserver> _observer;
     };
 
-    stdx::mutex _mutex;
+    Mutex _mutex = MONGO_MAKE_LATCH(HierarchicalAcquisitionLevel(2), "ServiceContext::_mutex");
 
     /**
      * The periodic runner.
@@ -611,6 +629,15 @@ bool hasGlobalServiceContext();
  */
 ServiceContext* getGlobalServiceContext();
 
+/**
+ * Returns the ServiceContext associated with the current Client.
+ *
+ * Returns a nullptr if there is not a current Client
+ *
+ * Caller does not own pointer.
+ */
+ServiceContext* getCurrentServiceContext();
+
 /**
  * Sets the global ServiceContext.  If 'serviceContext' is NULL, un-sets and deletes
  * the current global ServiceContext.
@@ -626,9 +653,4 @@ void setGlobalServiceContext(ServiceContext::UniqueServiceContext&& serviceConte
  */
 bool supportsDocLocking();
 
-/**
- * Returns true if the storage engine in use is MMAPV1.
- */
-bool isMMAPV1();
-
 }  // namespace mongo
