@@ -1,6 +1,3 @@
-// wiredtiger_server_status.cpp
-
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -45,7 +42,6 @@
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/scopeguard.h"
 
 
 namespace mongo {
@@ -73,8 +69,12 @@ BSONObj WiredTigerServerStatusSection::generateSection(OperationContext* opCtx,
     invariant(s);
     const string uri = "statistics:";
 
+    // Filter out unrelevant statistic fields.
+    std::vector<std::string> fieldsToIgnore = {"LSM"};
+
     BSONObjBuilder bob;
-    Status status = WiredTigerUtil::exportTableToBSON(s, uri, "statistics=(fast)", &bob);
+    Status status =
+        WiredTigerUtil::exportTableToBSON(s, uri, "statistics=(fast)", &bob, fieldsToIgnore);
     if (!status.isOK()) {
         bob.append("error", "unable to retrieve statistics");
         bob.append("code", static_cast<int>(status.code()));
@@ -83,6 +83,8 @@ BSONObj WiredTigerServerStatusSection::generateSection(OperationContext* opCtx,
 
     WiredTigerKVEngine::appendGlobalStats(bob);
 
+    WiredTigerUtil::appendSnapshotWindowSettings(_engine, session, &bob);
+
     {
         BSONObjBuilder subsection(bob.subobjStart("oplog"));
         subsection.append("visibility timestamp",
