@@ -1,6 +1,3 @@
-// wiredtiger_session_cache.cpp
-
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -41,9 +38,9 @@
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/global_settings.h"
 #include "mongo/db/repl/repl_settings.h"
-#include "mongo/db/server_parameters.h"
 #include "mongo/db/storage/journal_listener.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
+#include "mongo/db/storage/wiredtiger/wiredtiger_parameters_gen.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
 #include "mongo/stdx/memory.h"
 #include "mongo/stdx/thread.h"
@@ -52,42 +49,10 @@
 
 namespace mongo {
 
-// The "wiredTigerCursorCacheSize" parameter has the following meaning.
-//
-// wiredTigerCursorCacheSize == 0
-// For this setting, cursors are only cached in the WiredTiger storage engine
-// itself. Operations that need exclusive access such as drop or verify will
-// not be blocked by inactive cached cursors with this setting. However, this
-// setting may reduce the performance of certain workloads that normally
-// benefit from cursor caching above the storage engine.
-//
-// wiredTigerCursorCacheSize > 0
-// WiredTiger-level caching of cursors is disabled but cursor caching does
-// occur above the storage engine. The value of this setting represents the
-// maximum number of cursors that are cached. Setting the value to 10000 will
-// give the old (<= 3.6) behavior. Note that cursors remain cached, even when a
-// session is released back to the cache. Thus, exclusive operations may be
-// blocked temporarily, and in some cases, a long time. Drops that fail because
-// of exclusivity silently succeed and are queued for retries.
-//
-// wiredTigerCursorCacheSize < 0
-// This is a hybrid approach of the above two, and is the default. The the
-// absolute value of the setting is used as the number of cursors cached above
-// the storage engine. When a session is released, all cursors are closed, and
-// will be cached in WiredTiger. Exclusive operations should only be blocked
-// for a short time, except if a cursor is held by a long running session. This
-// is a good compromise for most workloads.
-AtomicInt32 kWiredTigerCursorCacheSize(-100);
-
 const std::string kWTRepairMsg =
     "Please read the documentation for starting MongoDB with --repair here: "
     "http://dochub.mongodb.org/core/repair";
 
-ExportedServerParameter<std::int32_t, ServerParameterType::kStartupAndRuntime>
-    WiredTigerCursorCacheSizeSetting(ServerParameterSet::getGlobal(),
-                                     "wiredTigerCursorCacheSize",
-                                     &kWiredTigerCursorCacheSize);
-
 WiredTigerSession::WiredTigerSession(WT_CONNECTION* conn, uint64_t epoch, uint64_t cursorEpoch)
     : _epoch(epoch),
       _cursorEpoch(cursorEpoch),
@@ -118,7 +83,29 @@ WiredTigerSession::~WiredTigerSession() {
     }
 }
 
-WT_CURSOR* WiredTigerSession::getCursor(const std::string& uri, uint64_t id, bool forRecordStore) {
+namespace {
+void _openCursor(WT_SESSION* session,
+                 const std::string& uri,
+                 const char* config,
+                 WT_CURSOR** cursorOut) {
+    int ret = session->open_cursor(session, uri.c_str(), NULL, config, cursorOut);
+    if (ret == EBUSY) {
+        // This can only happen when trying to open a cursor on the oplog and it is currently locked
+        // by a verify or salvage, because we don't employ database locks to protect the oplog.
+        throw WriteConflictException();
+    }
+    if (ret != 0) {
+        error() << "Failed to open a WiredTiger cursor. Reason: " << wtRCToStatus(ret)
+                << ", uri: " << uri << ", config: " << config;
+        error() << "This may be due to data corruption. " << kWTRepairMsg;
+
+        fassertFailedNoTrace(50882);
+    }
+}
+}  // namespace
+
+
+WT_CURSOR* WiredTigerSession::getCursor(const std::string& uri, uint64_t id, bool allowOverwrite) {
     // Find the most recently used cursor
     for (CursorCache::iterator i = _cursors.begin(); i != _cursors.end(); ++i) {
         if (i->_id == id) {
@@ -129,22 +116,19 @@ WT_CURSOR* WiredTigerSession::getCursor(const std::string& uri, uint64_t id, boo
         }
     }
 
-    WT_CURSOR* c = NULL;
-    int ret = _session->open_cursor(
-        _session, uri.c_str(), NULL, forRecordStore ? "" : "overwrite=false", &c);
-    if (ret == EBUSY) {
-        // This can only happen when trying to open a cursor on the oplog and it is currently locked
-        // by a verify or salvage, because we don't employ database locks to protect the oplog.
-        throw WriteConflictException();
-    }
-    if (ret != 0) {
-        error() << "Failed to open a WiredTiger cursor: " << uri;
-        error() << "This may be due to data corruption. " << kWTRepairMsg;
-
-        fassertFailedNoTrace(50882);
-    }
+    WT_CURSOR* cursor = NULL;
+    _openCursor(_session, uri, allowOverwrite ? "" : "overwrite=false", &cursor);
     _cursorsOut++;
-    return c;
+    return cursor;
+}
+
+WT_CURSOR* WiredTigerSession::getReadOnceCursor(const std::string& uri, bool allowOverwrite) {
+    const char* config = allowOverwrite ? "read_once=true" : "read_once=true,overwrite=false";
+
+    WT_CURSOR* cursor = NULL;
+    _openCursor(_session, uri, config, &cursor);
+    _cursorsOut++;
+    return cursor;
 }
 
 void WiredTigerSession::releaseCursor(uint64_t id, WT_CURSOR* cursor) {
@@ -158,7 +142,7 @@ void WiredTigerSession::releaseCursor(uint64_t id, WT_CURSOR* cursor) {
     _cursors.push_front(WiredTigerCachedCursor(id, _cursorGen++, cursor));
 
     // A negative value for wiredTigercursorCacheSize means to use hybrid caching.
-    std::uint32_t cacheSize = abs(kWiredTigerCursorCacheSize.load());
+    std::uint32_t cacheSize = abs(gWiredTigerCursorCacheSize.load());
 
     while (!_cursors.empty() && _cursorGen - _cursors.back()._gen > cacheSize) {
         cursor = _cursors.back()._cursor;
@@ -167,6 +151,14 @@ void WiredTigerSession::releaseCursor(uint64_t id, WT_CURSOR* cursor) {
     }
 }
 
+void WiredTigerSession::closeCursor(WT_CURSOR* cursor) {
+    invariant(_session);
+    invariant(cursor);
+    _cursorsOut--;
+
+    invariantWTOK(cursor->close(cursor));
+}
+
 void WiredTigerSession::closeAllCursors(const std::string& uri) {
     invariant(_session);
 
@@ -196,7 +188,7 @@ void WiredTigerSession::closeCursorsForQueuedDrops(WiredTigerKVEngine* engine) {
 }
 
 namespace {
-AtomicUInt64 nextTableId(WiredTigerSession::kLastTableId);
+AtomicWord<unsigned long long> nextTableId(WiredTigerSession::kLastTableId);
 }
 // static
 uint64_t WiredTigerSession::genTableId() {
@@ -209,26 +201,24 @@ WiredTigerSessionCache::WiredTigerSessionCache(WiredTigerKVEngine* engine)
     : _engine(engine),
       _conn(engine->getConnection()),
       _clockSource(_engine->getClockSource()),
-      _shuttingDown(0) {}
+      _shuttingDown(0),
+      _prepareCommitOrAbortCounter(0) {}
 
 WiredTigerSessionCache::WiredTigerSessionCache(WT_CONNECTION* conn, ClockSource* cs)
-    : _engine(NULL), _conn(conn), _clockSource(cs), _shuttingDown(0) {}
+    : _engine(nullptr),
+      _conn(conn),
+      _clockSource(cs),
+      _shuttingDown(0),
+      _prepareCommitOrAbortCounter(0) {}
 
 WiredTigerSessionCache::~WiredTigerSessionCache() {
     shuttingDown();
 }
 
 void WiredTigerSessionCache::shuttingDown() {
-    uint32_t actual = _shuttingDown.load();
-    uint32_t expected;
-
     // Try to atomically set _shuttingDown flag, but just return if another thread was first.
-    do {
-        expected = actual;
-        actual = _shuttingDown.compareAndSwap(expected, expected | kShuttingDownMask);
-        if (actual & kShuttingDownMask)
-            return;
-    } while (actual != expected);
+    if (_shuttingDown.fetchAndBitOr(kShuttingDownMask) & kShuttingDownMask)
+        return;
 
     // Spin as long as there are threads in releaseSession
     while (_shuttingDown.load() != kShuttingDownMask) {
@@ -245,10 +235,6 @@ void WiredTigerSessionCache::waitUntilDurable(bool forceCheckpoint, bool stableC
         return;
     }
 
-    if (stableCheckpoint) {
-        invariant(serverGlobalParams.enableMajorityReadConcern);
-    }
-
     const int shuttingDown = _shuttingDown.fetchAndAdd(1);
     ON_BLOCK_EXIT([this] { _shuttingDown.fetchAndSubtract(1); });
 
@@ -278,7 +264,7 @@ void WiredTigerSessionCache::waitUntilDurable(bool forceCheckpoint, bool stableC
             s2 = session2->getSession();
         }
         {
-            stdx::unique_lock<stdx::mutex> lk(_journalListenerMutex);
+            stdx::unique_lock<Latch> lk(_journalListenerMutex);
             JournalListener::Token token = _journalListener->getToken();
 #ifdef PERCONA_AUDIT_ENABLED
             // Make audit log durable
@@ -297,7 +283,7 @@ void WiredTigerSessionCache::waitUntilDurable(bool forceCheckpoint, bool stableC
     uint32_t start = _lastSyncTime.load();
     // Do the remainder in a critical section that ensures only a single thread at a time
     // will attempt to synchronize.
-    stdx::unique_lock<stdx::mutex> lk(_lastSyncMutex);
+    stdx::unique_lock<Latch> lk(_lastSyncMutex);
     uint32_t current = _lastSyncTime.loadRelaxed();  // synchronized with writes through mutex
     if (current != start) {
         // Someone else synced already since we read lastSyncTime, so we're done!
@@ -309,7 +295,7 @@ void WiredTigerSessionCache::waitUntilDurable(bool forceCheckpoint, bool stableC
 
     // This gets the token (OpTime) from the last write, before flushing (either the journal, or a
     // checkpoint), and then reports that token (OpTime) as a durable write.
-    stdx::unique_lock<stdx::mutex> jlk(_journalListenerMutex);
+    stdx::unique_lock<Latch> jlk(_journalListenerMutex);
     JournalListener::Token token = _journalListener->getToken();
 
     // Initialize on first use.
@@ -348,27 +334,26 @@ void WiredTigerSessionCache::waitUntilDurable(bool forceCheckpoint, bool stableC
     _journalListener->onDurable(token);
 }
 
-void WiredTigerSessionCache::waitUntilPreparedUnitOfWorkCommitsOrAborts(OperationContext* opCtx) {
+void WiredTigerSessionCache::waitUntilPreparedUnitOfWorkCommitsOrAborts(OperationContext* opCtx,
+                                                                        std::uint64_t lastCount) {
     invariant(opCtx);
-    stdx::unique_lock<stdx::mutex> lk(_prepareCommittedOrAbortedMutex);
-
-    auto lastCounter = _lastCommitOrAbortCounter;
-    opCtx->waitForConditionOrInterrupt(_prepareCommittedOrAbortedCond, lk, [&] {
-        return lastCounter != _lastCommitOrAbortCounter;
-    });
+    stdx::unique_lock<Latch> lk(_prepareCommittedOrAbortedMutex);
+    if (lastCount == _prepareCommitOrAbortCounter.loadRelaxed()) {
+        opCtx->waitForConditionOrInterrupt(_prepareCommittedOrAbortedCond, lk, [&] {
+            return _prepareCommitOrAbortCounter.loadRelaxed() > lastCount;
+        });
+    }
 }
 
 void WiredTigerSessionCache::notifyPreparedUnitOfWorkHasCommittedOrAborted() {
-    {
-        stdx::unique_lock<stdx::mutex> lk(_prepareCommittedOrAbortedMutex);
-        _lastCommitOrAbortCounter++;
-    }
+    stdx::unique_lock<Latch> lk(_prepareCommittedOrAbortedMutex);
+    _prepareCommitOrAbortCounter.fetchAndAdd(1);
     _prepareCommittedOrAbortedCond.notify_all();
 }
 
 
 void WiredTigerSessionCache::closeAllCursors(const std::string& uri) {
-    stdx::lock_guard<stdx::mutex> lock(_cacheLock);
+    stdx::lock_guard<Latch> lock(_cacheLock);
     for (SessionCache::iterator i = _sessions.begin(); i != _sessions.end(); i++) {
         (*i)->closeAllCursors(uri);
     }
@@ -378,14 +363,14 @@ void WiredTigerSessionCache::closeCursorsForQueuedDrops() {
     // Increment the cursor epoch so that all cursors from this epoch are closed.
     _cursorEpoch.fetchAndAdd(1);
 
-    stdx::lock_guard<stdx::mutex> lock(_cacheLock);
+    stdx::lock_guard<Latch> lock(_cacheLock);
     for (SessionCache::iterator i = _sessions.begin(); i != _sessions.end(); i++) {
         (*i)->closeCursorsForQueuedDrops(_engine);
     }
 }
 
 size_t WiredTigerSessionCache::getIdleSessionsCount() {
-    stdx::lock_guard<stdx::mutex> lock(_cacheLock);
+    stdx::lock_guard<Latch> lock(_cacheLock);
     return _sessions.size();
 }
 
@@ -399,7 +384,7 @@ void WiredTigerSessionCache::closeExpiredIdleSessions(int64_t idleTimeMillis) {
     SessionCache sessionsToClose;
 
     {
-        stdx::lock_guard<stdx::mutex> lock(_cacheLock);
+        stdx::lock_guard<Latch> lock(_cacheLock);
         // Discard all sessions that became idle before the cutoff time
         for (auto it = _sessions.begin(); it != _sessions.end();) {
             auto session = *it;
@@ -425,7 +410,7 @@ void WiredTigerSessionCache::closeAll() {
     SessionCache swap;
 
     {
-        stdx::lock_guard<stdx::mutex> lock(_cacheLock);
+        stdx::lock_guard<Latch> lock(_cacheLock);
         _epoch.fetchAndAdd(1);
         _sessions.swap(swap);
     }
@@ -445,7 +430,7 @@ UniqueWiredTigerSession WiredTigerSessionCache::getSession() {
     invariant(!(_shuttingDown.loadRelaxed() & kShuttingDownMask));
 
     {
-        stdx::lock_guard<stdx::mutex> lock(_cacheLock);
+        stdx::lock_guard<Latch> lock(_cacheLock);
         if (!_sessions.empty()) {
             // Get the most recently used session so that if we discard sessions, we're
             // discarding older ones
@@ -491,7 +476,7 @@ void WiredTigerSessionCache::releaseSession(WiredTigerSession* session) {
         // Release resources in the session we're about to cache.
         // If we are using hybrid caching, then close cursors now and let them
         // be cached at the WiredTiger level.
-        if (kWiredTigerCursorCacheSize.load() < 0) {
+        if (gWiredTigerCursorCacheSize.load() < 0) {
             session->closeAllCursors("");
         }
         invariantWTOK(ss->reset(ss));
@@ -512,7 +497,7 @@ void WiredTigerSessionCache::releaseSession(WiredTigerSession* session) {
     session->setIdleExpireTime(_clockSource->now());
 
     if (session->_getEpoch() == currentEpoch) {  // check outside of lock to reduce contention
-        stdx::lock_guard<stdx::mutex> lock(_cacheLock);
+        stdx::lock_guard<Latch> lock(_cacheLock);
         if (session->_getEpoch() == _epoch.load()) {  // recheck inside the lock for correctness
             returnedToCache = true;
             _sessions.push_back(session);
@@ -529,12 +514,12 @@ void WiredTigerSessionCache::releaseSession(WiredTigerSession* session) {
 
 
 void WiredTigerSessionCache::setJournalListener(JournalListener* jl) {
-    stdx::unique_lock<stdx::mutex> lk(_journalListenerMutex);
+    stdx::unique_lock<Latch> lk(_journalListenerMutex);
     _journalListener = jl;
 }
 
 bool WiredTigerSessionCache::isEngineCachingCursors() {
-    return kWiredTigerCursorCacheSize.load() <= 0;
+    return gWiredTigerCursorCacheSize.load() <= 0;
 }
 
 void WiredTigerSessionCache::WiredTigerSessionDeleter::operator()(
