@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -30,16 +29,15 @@
 
 #pragma once
 
-#include "mongo/base/disallow_copying.h"
 #include "mongo/base/string_data.h"
 #include "mongo/platform/atomic_word.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/s/catalog/type_database.h"
 #include "mongo/s/catalog_cache_loader.h"
 #include "mongo/s/chunk_manager.h"
 #include "mongo/s/client/shard.h"
 #include "mongo/s/database_version_gen.h"
 #include "mongo/stdx/memory.h"
-#include "mongo/stdx/mutex.h"
 #include "mongo/util/concurrency/notification.h"
 #include "mongo/util/concurrency/with_lock.h"
 #include "mongo/util/string_map.h"
@@ -65,7 +63,7 @@ public:
     };
 
     bool shardingEnabled() const;
-    boost::optional<DatabaseVersion> databaseVersion() const;
+    DatabaseVersion databaseVersion() const;
 
 private:
     friend class CatalogCache;
@@ -119,7 +117,8 @@ private:
  * writes happen through the ShardingCatalogManager and the cache hierarchy needs to be invalidated.
  */
 class CatalogCache {
-    MONGO_DISALLOW_COPYING(CatalogCache);
+    CatalogCache(const CatalogCache&) = delete;
+    CatalogCache& operator=(const CatalogCache&) = delete;
 
 public:
     CatalogCache(CatalogCacheLoader& cacheLoader);
@@ -213,6 +212,15 @@ public:
      */
     void onStaleShardVersion(CachedCollectionRoutingInfo&&);
 
+    /**
+     * Throws a StaleConfigException if this catalog cache does not have an entry for the given
+     * namespace, or if the entry for the given namespace does not have the same epoch as
+     * 'targetCollectionVersion'. Does not perform any refresh logic. Ignores everything except the
+     * epoch of 'targetCollectionVersion' when performing the check, but needs the entire target
+     * version to throw a StaleConfigException.
+     */
+    void checkEpochOrThrow(const NamespaceString& nss, ChunkVersion targetCollectionVersion) const;
+
     /**
      * Non-blocking method, which indiscriminately causes the database entry for the specified
      * database to be refreshed the next time getDatabase is called.
@@ -225,6 +233,12 @@ public:
      */
     void invalidateShardedCollection(const NamespaceString& nss);
 
+    /**
+     * Non-blocking method, which invalidates all namespaces which contain data on the specified
+     * shard and all databases which have the shard listed as their primary shard.
+     */
+    void invalidateEntriesThatReferenceShard(const ShardId& shardId);
+
     /**
      * Non-blocking method, which removes the entire specified collection from the cache (resulting
      * in full refresh on subsequent access)
@@ -248,6 +262,12 @@ public:
      */
     void report(BSONObjBuilder* builder) const;
 
+    /**
+     * Checks if the current operation was ever marked as needing refresh. If the curent operation
+     * was marked as needing refresh, updates the relevant counters inside the Stats struct.
+     */
+    void checkAndRecordOperationBlockedByRefresh(OperationContext* opCtx, mongo::LogicalOp opType);
+
 private:
     // Make the cache entries friends so they can access the private classes below
     friend class CachedDatabaseInfo;
@@ -351,28 +371,40 @@ private:
     struct Stats {
         // Counts how many times threads hit stale config exception (which is what triggers metadata
         // refreshes)
-        AtomicInt64 countStaleConfigErrors{0};
+        AtomicWord<long long> countStaleConfigErrors{0};
 
         // Cumulative, always-increasing counter of how much time threads waiting for refresh
         // combined
-        AtomicInt64 totalRefreshWaitTimeMicros{0};
+        AtomicWord<long long> totalRefreshWaitTimeMicros{0};
 
         // Tracks how many incremental refreshes are waiting to complete currently
-        AtomicInt64 numActiveIncrementalRefreshes{0};
+        AtomicWord<long long> numActiveIncrementalRefreshes{0};
 
         // Cumulative, always-increasing counter of how many incremental refreshes have been kicked
         // off
-        AtomicInt64 countIncrementalRefreshesStarted{0};
+        AtomicWord<long long> countIncrementalRefreshesStarted{0};
 
         // Tracks how many full refreshes are waiting to complete currently
-        AtomicInt64 numActiveFullRefreshes{0};
+        AtomicWord<long long> numActiveFullRefreshes{0};
 
         // Cumulative, always-increasing counter of how many full refreshes have been kicked off
-        AtomicInt64 countFullRefreshesStarted{0};
+        AtomicWord<long long> countFullRefreshesStarted{0};
 
         // Cumulative, always-increasing counter of how many full or incremental refreshes failed
         // for whatever reason
-        AtomicInt64 countFailedRefreshes{0};
+        AtomicWord<long long> countFailedRefreshes{0};
+
+        // Cumulative, always-increasing counter of how many operations have been blocked by a
+        // catalog cache refresh. Broken down by operation type to match the operations tracked
+        // by the OpCounters class.
+        struct OperationsBlockedByRefresh {
+            AtomicWord<long long> countAllOperations{0};
+            AtomicWord<long long> countInserts{0};
+            AtomicWord<long long> countQueries{0};
+            AtomicWord<long long> countUpdates{0};
+            AtomicWord<long long> countDeletes{0};
+            AtomicWord<long long> countCommands{0};
+        } operationsBlockedByRefresh;
 
         /**
          * Reports the accumulated statistics for serverStatus.
@@ -386,7 +418,7 @@ private:
     using CollectionsByDbMap = StringMap<CollectionInfoMap>;
 
     // Mutex to serialize access to the structures below
-    mutable stdx::mutex _mutex;
+    mutable Mutex _mutex = MONGO_MAKE_LATCH("CatalogCache::_mutex");
 
     // Map from DB name to the info for that database
     DatabaseInfoMap _databases;
