@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -55,15 +54,15 @@
 #include "mongo/s/catalog_cache.h"
 #include "mongo/s/client/num_hosts_targeted_metrics.h"
 #include "mongo/s/client/shard_registry.h"
-#include "mongo/s/commands/cluster_commands_helpers.h"
+#include "mongo/s/cluster_commands_helpers.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/query/async_results_merger.h"
 #include "mongo/s/query/cluster_client_cursor_impl.h"
 #include "mongo/s/query/cluster_cursor_manager.h"
-#include "mongo/s/query/cluster_query_knobs.h"
 #include "mongo/s/query/establish_cursors.h"
 #include "mongo/s/query/store_possible_cursor.h"
 #include "mongo/s/stale_exception.h"
+#include "mongo/s/transaction_router.h"
 #include "mongo/stdx/memory.h"
 #include "mongo/util/fail_point_service.h"
 #include "mongo/util/log.h"
@@ -84,6 +83,8 @@ static const BSONObj kGeoNearDistanceMetaProjection = BSON("$meta"
 // more than 8 decimal digits since the response is at most 16MB, and 16 * 1024 * 1024 < 1 * 10^8.
 static const int kPerDocumentOverheadBytesUpperBound = 10;
 
+const char kFindCmdName[] = "find";
+
 /**
  * Given the QueryRequest 'qr' being executed by mongos, returns a copy of the query which is
  * suitable for forwarding to the targeted hosts.
@@ -99,9 +100,7 @@ StatusWith<std::unique_ptr<QueryRequest>> transformQueryForShards(
                 ErrorCodes::Overflow,
                 str::stream()
                     << "sum of limit and skip cannot be represented as a 64-bit integer, limit: "
-                    << *qr.getLimit()
-                    << ", skip: "
-                    << qr.getSkip().value_or(0));
+                    << *qr.getLimit() << ", skip: " << qr.getSkip().value_or(0));
         }
         newLimit = newLimitValue;
     }
@@ -118,9 +117,7 @@ StatusWith<std::unique_ptr<QueryRequest>> transformQueryForShards(
                               str::stream()
                                   << "sum of ntoreturn and skip cannot be represented as a 64-bit "
                                      "integer, ntoreturn: "
-                                  << *qr.getNToReturn()
-                                  << ", skip: "
-                                  << qr.getSkip().value_or(0));
+                                  << *qr.getNToReturn() << ", skip: " << qr.getSkip().value_or(0));
             }
             newLimit = newLimitValue;
         } else {
@@ -131,9 +128,7 @@ StatusWith<std::unique_ptr<QueryRequest>> transformQueryForShards(
                               str::stream()
                                   << "sum of ntoreturn and skip cannot be represented as a 64-bit "
                                      "integer, ntoreturn: "
-                                  << *qr.getNToReturn()
-                                  << ", skip: "
-                                  << qr.getSkip().value_or(0));
+                                  << *qr.getNToReturn() << ", skip: " << qr.getSkip().value_or(0));
             }
             newNToReturn = newNToReturnValue;
         }
@@ -180,17 +175,10 @@ std::vector<std::pair<ShardId, BSONObj>> constructRequestsForShards(
     const CachedCollectionRoutingInfo& routingInfo,
     const std::set<ShardId>& shardIds,
     const CanonicalQuery& query,
-    bool appendGeoNearDistanceProjection,
-    boost::optional<LogicalTime> atClusterTime) {
+    bool appendGeoNearDistanceProjection) {
     const auto qrToForward = uassertStatusOK(
         transformQueryForShards(query.getQueryRequest(), appendGeoNearDistanceProjection));
 
-    if (atClusterTime) {
-        auto readConcernAtClusterTime =
-            appendAtClusterTimeToReadConcern(qrToForward->getReadConcern(), *atClusterTime);
-        qrToForward->setReadConcern(readConcernAtClusterTime);
-    }
-
     auto shardRegistry = Grid::get(opCtx)->shardRegistry();
     std::vector<std::pair<ShardId, BSONObj>> requests;
     for (const auto& shardId : shardIds) {
@@ -206,7 +194,6 @@ std::vector<std::pair<ShardId, BSONObj>> constructRequestsForShards(
             ChunkVersion::UNSHARDED().appendToCommand(&cmdBuilder);
         }
 
-        // TODO SERVER-33702: standardize method for attaching txnNumber through mongos.
         if (opCtx->getTxnNumber()) {
             cmdBuilder.append(OperationSessionInfo::kTxnNumberFieldName, *opCtx->getTxnNumber());
         }
@@ -242,16 +229,6 @@ CursorId runQueryWithoutRetrying(OperationContext* opCtx,
                                               query.getQueryRequest().getFilter(),
                                               query.getQueryRequest().getCollation());
 
-    // Determine atClusterTime for snapshot reads. This will be a null time for requests with any
-    // other readConcern.
-    auto atClusterTime = computeAtClusterTime(opCtx,
-                                              false,
-                                              shardIds,
-                                              query.nss(),
-                                              query.getQueryRequest().getFilter(),
-                                              query.getQueryRequest().getCollation());
-
-    invariant(!atClusterTime || *atClusterTime != LogicalTime::kUninitialized);
     // Construct the query and parameters.
 
     ClusterClientCursorParams params(query.nss(), readPref);
@@ -263,6 +240,12 @@ CursorId runQueryWithoutRetrying(OperationContext* opCtx,
     params.isAllowPartialResults = query.getQueryRequest().isAllowPartialResults();
     params.lsid = opCtx->getLogicalSessionId();
     params.txnNumber = opCtx->getTxnNumber();
+    params.originatingPrivileges = {
+        Privilege(ResourcePattern::forExactNamespace(query.nss()), ActionType::find)};
+
+    if (TransactionRouter::get(opCtx)) {
+        params.isAutoCommit = false;
+    }
 
     // This is the batchSize passed to each subsequent getMore command issued by the cursor. We
     // usually use the batchSize associated with the initial find, but as it is illegal to send a
@@ -297,7 +280,7 @@ CursorId runQueryWithoutRetrying(OperationContext* opCtx,
     // attaching the shardVersion and txnNumber, if necessary.
 
     auto requests = constructRequestsForShards(
-        opCtx, routingInfo, shardIds, query, appendGeoNearDistanceProjection, atClusterTime);
+        opCtx, routingInfo, shardIds, query, appendGeoNearDistanceProjection);
 
     // Establish the cursors with a consistent shardVersion across shards.
 
@@ -321,8 +304,7 @@ CursorId runQueryWithoutRetrying(OperationContext* opCtx,
 
     // Retrieve enough data from the ClusterClientCursor for the first batch of results.
 
-    CurOpFailpointHelpers::waitWhileFailPointEnabled(
-        &waitInFindBeforeMakingBatch, opCtx, "waitInFindBeforeMakingBatch");
+    FindCommon::waitInFindBeforeMakingBatch(opCtx, query);
 
     auto cursorState = ClusterCursorManager::CursorState::NotExhausted;
     int bytesBuffered = 0;
@@ -384,6 +366,7 @@ CursorId runQueryWithoutRetrying(OperationContext* opCtx,
         ? ClusterCursorManager::CursorLifetime::Immortal
         : ClusterCursorManager::CursorLifetime::Mortal;
     auto authUsers = AuthorizationSession::get(opCtx->getClient())->getAuthenticatedUserNames();
+    ccc->incNBatches();
 
     auto cursorId = uassertStatusOK(cursorManager->registerCursor(
         opCtx, ccc.releaseCursor(), query.nss(), cursorType, cursorLifetime, authUsers));
@@ -444,8 +427,7 @@ CursorId ClusterFind::runQuery(OperationContext* opCtx,
         uasserted(ErrorCodes::BadValue,
                   str::stream() << "Projection contains illegal field '"
                                 << AsyncResultsMerger::kSortKeyField
-                                << "': "
-                                << query.getQueryRequest().getProj());
+                                << "': " << query.getQueryRequest().getProj());
     }
 
     auto const catalogCache = Grid::get(opCtx)->catalogCache();
@@ -453,7 +435,7 @@ CursorId ClusterFind::runQuery(OperationContext* opCtx,
     // Re-target and re-send the initial find command to the shards until we have established the
     // shard version.
     for (size_t retries = 1; retries <= kMaxRetries; ++retries) {
-        auto routingInfoStatus = catalogCache->getCollectionRoutingInfo(opCtx, query.nss());
+        auto routingInfoStatus = getCollectionRoutingInfoForTxnCmd(opCtx, query.nss());
         if (routingInfoStatus == ErrorCodes::NamespaceNotFound) {
             // If the database doesn't exist, we successfully return an empty result set without
             // creating a cursor.
@@ -468,15 +450,14 @@ CursorId ClusterFind::runQuery(OperationContext* opCtx,
             if (retries >= kMaxRetries) {
                 // Check if there are no retries remaining, so the last received error can be
                 // propagated to the caller.
-                ex.addContext(str::stream() << "Failed to run query after " << kMaxRetries
-                                            << " retries");
+                ex.addContext(str::stream()
+                              << "Failed to run query after " << kMaxRetries << " retries");
                 throw;
             } else if (!ErrorCodes::isStaleShardVersionError(ex.code()) &&
-                       !ErrorCodes::isSnapshotError(ex.code()) &&
                        ex.code() != ErrorCodes::ShardNotFound) {
-                // Errors other than stale metadata, snapshot unavailable, or from trying to reach a
-                // non existent shard are fatal to the operation. Network errors and replication
-                // retries happen at the level of the AsyncResultsMerger.
+                // Errors other than stale metadata or from trying to reach a non existent shard are
+                // fatal to the operation. Network errors and replication retries happen at the
+                // level of the AsyncResultsMerger.
                 ex.addContext("Encountered non-retryable error during query");
                 throw;
             }
@@ -484,12 +465,18 @@ CursorId ClusterFind::runQuery(OperationContext* opCtx,
             LOG(1) << "Received error status for query " << redact(query.toStringShort())
                    << " on attempt " << retries << " of " << kMaxRetries << ": " << redact(ex);
 
-            // Note: there is no need to refresh metadata on snapshot errors since the request
-            // failed because atClusterTime was too low, not because the wrong shards were targeted,
-            // and subsequent attempts will choose a later atClusterTime.
-            if (ErrorCodes::isStaleShardVersionError(ex.code()) ||
-                ex.code() == ErrorCodes::ShardNotFound) {
-                catalogCache->onStaleShardVersion(std::move(routingInfo));
+            catalogCache->onStaleShardVersion(std::move(routingInfo));
+
+            if (auto txnRouter = TransactionRouter::get(opCtx)) {
+                if (!txnRouter.canContinueOnStaleShardOrDbError(kFindCmdName)) {
+                    throw;
+                }
+
+                // Reset the default global read timestamp so the retry's routing table reflects the
+                // chunk placement after the refresh (no-op if the transaction is not running with
+                // snapshot read concern).
+                txnRouter.onStaleShardOrDbError(opCtx, kFindCmdName, ex.toStatus());
+                txnRouter.setDefaultAtClusterTime(opCtx);
             }
         }
     }
@@ -514,8 +501,7 @@ void validateLSID(OperationContext* opCtx,
     if (!opCtx->getLogicalSessionId() && cursor->getLsid()) {
         uasserted(50800,
                   str::stream() << "Cannot run getMore on cursor " << request.cursorid
-                                << ", which was created in session "
-                                << *cursor->getLsid()
+                                << ", which was created in session " << *cursor->getLsid()
                                 << ", without an lsid");
     }
 
@@ -523,10 +509,8 @@ void validateLSID(OperationContext* opCtx,
         (*opCtx->getLogicalSessionId() != *cursor->getLsid())) {
         uasserted(50801,
                   str::stream() << "Cannot run getMore on cursor " << request.cursorid
-                                << ", which was created in session "
-                                << *cursor->getLsid()
-                                << ", in session "
-                                << *opCtx->getLogicalSessionId());
+                                << ", which was created in session " << *cursor->getLsid()
+                                << ", in session " << *opCtx->getLogicalSessionId());
     }
 }
 
@@ -547,8 +531,7 @@ void validateTxnNumber(OperationContext* opCtx,
     if (!opCtx->getTxnNumber() && cursor->getTxnNumber()) {
         uasserted(50803,
                   str::stream() << "Cannot run getMore on cursor " << request.cursorid
-                                << ", which was created in transaction "
-                                << *cursor->getTxnNumber()
+                                << ", which was created in transaction " << *cursor->getTxnNumber()
                                 << ", without a txnNumber");
     }
 
@@ -556,10 +539,8 @@ void validateTxnNumber(OperationContext* opCtx,
         (*opCtx->getTxnNumber() != *cursor->getTxnNumber())) {
         uasserted(50804,
                   str::stream() << "Cannot run getMore on cursor " << request.cursorid
-                                << ", which was created in transaction "
-                                << *cursor->getTxnNumber()
-                                << ", in transaction "
-                                << *opCtx->getTxnNumber());
+                                << ", which was created in transaction " << *cursor->getTxnNumber()
+                                << ", in transaction " << *opCtx->getTxnNumber());
     }
 }
 
@@ -570,11 +551,11 @@ void validateTxnNumber(OperationContext* opCtx,
 void validateOperationSessionInfo(OperationContext* opCtx,
                                   const GetMoreRequest& request,
                                   ClusterCursorManager::PinnedCursor* cursor) {
-    ScopeGuard returnCursorGuard = MakeGuard(
+    auto returnCursorGuard = makeGuard(
         [cursor] { cursor->returnCursor(ClusterCursorManager::CursorState::NotExhausted); });
     validateLSID(opCtx, request, cursor);
     validateTxnNumber(opCtx, request, cursor);
-    returnCursorGuard.Dismiss();
+    returnCursorGuard.dismiss();
 }
 
 StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* opCtx,
@@ -597,6 +578,14 @@ StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* opCtx,
 
     validateOperationSessionInfo(opCtx, request, &pinnedCursor.getValue());
 
+    // Ensure that the client still has the privileges to run the originating command.
+    if (!authzSession->isAuthorizedForPrivileges(
+            pinnedCursor.getValue().getOriginatingPrivileges())) {
+        uasserted(ErrorCodes::Unauthorized,
+                  str::stream() << "not authorized for getMore with cursor id "
+                                << request.cursorid);
+    }
+
     // Set the originatingCommand object and the cursorID in CurOp.
     {
         CurOp::get(opCtx)->debug().nShards = pinnedCursor.getValue().getNumRemotes();
@@ -604,6 +593,7 @@ StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* opCtx,
         stdx::lock_guard<Client> lk(*opCtx->getClient());
         CurOp::get(opCtx)->setOriginatingCommand_inlock(
             pinnedCursor.getValue().getOriginatingCommand());
+        CurOp::get(opCtx)->setGenericCursor_inlock(pinnedCursor.getValue().toGenericCursor());
     }
 
     // If the 'waitAfterPinningCursorBeforeGetMoreBatch' fail point is enabled, set the 'msg'
@@ -629,6 +619,14 @@ StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* opCtx,
     BSONObj postBatchResumeToken;
     bool stashedResult = false;
 
+    // If the 'waitWithPinnedCursorDuringGetMoreBatch' fail point is enabled, set the 'msg'
+    // field of this operation's CurOp to signal that we've hit this point.
+    if (MONGO_FAIL_POINT(waitWithPinnedCursorDuringGetMoreBatch)) {
+        CurOpFailpointHelpers::waitWhileFailPointEnabled(&waitWithPinnedCursorDuringGetMoreBatch,
+                                                         opCtx,
+                                                         "waitWithPinnedCursorDuringGetMoreBatch");
+    }
+
     while (!FindCommon::enoughForGetMore(batchSize, batch.size())) {
         auto context = batch.empty()
             ? RouterExecStage::ExecContext::kGetMoreNoResultsYet
@@ -691,6 +689,7 @@ StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* opCtx,
     }
 
     pinnedCursor.getValue().setLeftoverMaxTimeMicros(opCtx->getRemainingMaxTimeMicros());
+    pinnedCursor.getValue().incNBatches();
     // Upon successful completion, transfer ownership of the cursor back to the cursor manager. If
     // the cursor has been exhausted, the cursor manager will clean it up for us.
     pinnedCursor.getValue().returnCursor(cursorState);
