@@ -1,13 +1,13 @@
 doassert = function(msg, obj) {
     // eval if msg is a function
-    if (typeof(msg) == "function")
+    if (typeof (msg) == "function")
         msg = msg();
 
-    if (typeof(msg) == "object")
+    if (typeof (msg) == "object")
         msg = tojson(msg);
 
     if (jsTest.options().traceExceptions) {
-        if (typeof(msg) == "string" && msg.indexOf("assert") == 0)
+        if (typeof (msg) == "string" && msg.indexOf("assert") == 0)
             print(msg);
         else
             print("assert: " + msg);
@@ -27,7 +27,6 @@ doassert = function(msg, obj) {
 
 // Sort doc/obj fields and return new sorted obj
 sortDoc = function(doc) {
-
     // Helper to sort the elements of the array
     var sortElementsOfArray = function(arr) {
         var newArr = [];
@@ -318,7 +317,7 @@ assert = (function() {
         var msgPrefix = "assert.soon failed: " + func;
 
         if (msg) {
-            if (typeof(msg) != "function") {
+            if (typeof (msg) != "function") {
                 msgPrefix = "assert.soon failed, msg";
             }
         }
@@ -328,7 +327,7 @@ assert = (function() {
         interval = interval || 200;
         var last;
         while (1) {
-            if (typeof(func) == "string") {
+            if (typeof (func) == "string") {
                 if (eval(func))
                     return;
             } else {
@@ -439,7 +438,7 @@ assert = (function() {
 
         var start = new Date();
         timeout = timeout || 30000;
-        if (typeof(f) == "string") {
+        if (typeof (f) == "string") {
             res = eval(f);
         } else {
             res = f();
@@ -569,12 +568,16 @@ assert = (function() {
             res instanceof BulkWriteResult || res instanceof BulkWriteError;
     }
 
+    function _validateCommandResponse(res, assertionName) {
+        if (typeof res !== "object") {
+            doassert("unknown response type '" + typeof res + "' given to " + assertionName +
+                     ", res='" + res + "'");
+        }
+    }
+
     function _assertCommandWorked(res, msg, {ignoreWriteErrors, ignoreWriteConcernErrors}) {
         _validateAssertionMessage(msg);
-
-        if (typeof res !== "object") {
-            doassert("unknown response given to commandWorked");
-        }
+        _validateCommandResponse(res, "commandWorked");
 
         // Keep this as a function so we don't call tojson if not necessary.
         const makeFailMsg = () => {
@@ -614,10 +617,7 @@ assert = (function() {
     const kAnyErrorCode = Object.create(null);
     function _assertCommandFailed(res, expectedCode, msg) {
         _validateAssertionMessage(msg);
-
-        if (typeof res !== "object") {
-            doassert("unknown response given to commandFailed");
-        }
+        _validateCommandResponse(res, "commandFailed");
 
         if (expectedCode !== kAnyErrorCode && !Array.isArray(expectedCode)) {
             expectedCode = [expectedCode];
@@ -907,11 +907,7 @@ assert = (function() {
         assert.between(a, b, c, msg, false);
     };
 
-    assert.close = function(a, b, msg, places) {
-        if (places === undefined) {
-            places = 4;
-        }
-
+    assert.close = function(a, b, msg, places = 4) {
         // This treats 'places' as digits past the decimal point.
         var absoluteError = Math.abs(a - b);
         if (Math.round(absoluteError * Math.pow(10, places)) === 0) {
@@ -924,8 +920,8 @@ assert = (function() {
             return;
         }
 
-        const msgPrefix = "" + a + " is not equal to " + b + " within " + places + " places, " +
-            "absolute error: " + absoluteError + ", relative error: " + relativeError;
+        const msgPrefix = `${a} is not equal to ${b} within ${places} places, absolute error: ` +
+            `${absoluteError}, relative error: ${relativeError}`;
         doassert(_buildAssertionMessage(msg, msgPrefix));
     };
 
@@ -953,7 +949,6 @@ assert = (function() {
     };
 
     assert.gleOK = function(res, msg) {
-
         var errMsg = null;
 
         if (!res) {
@@ -974,7 +969,7 @@ assert = (function() {
     assert.gleSuccess = function(dbOrGLEDoc, msg) {
         var gle = dbOrGLEDoc instanceof DB ? dbOrGLEDoc.getLastErrorObj() : dbOrGLEDoc;
         if (gle.err) {
-            if (typeof(msg) == "function")
+            if (typeof (msg) == "function")
                 msg = msg(gle);
             doassert(_buildAssertionMessage(msg, "getLastError not null: " + tojson(gle)), gle);
         }
@@ -984,7 +979,7 @@ assert = (function() {
     assert.gleError = function(dbOrGLEDoc, msg) {
         var gle = dbOrGLEDoc instanceof DB ? dbOrGLEDoc.getLastErrorObj() : dbOrGLEDoc;
         if (!gle.err) {
-            if (typeof(msg) == "function")
+            if (typeof (msg) == "function")
                 msg = msg(gle);
             doassert(_buildAssertionMessage(msg, "getLastError is null: " + tojson(gle)));
         }
@@ -993,7 +988,7 @@ assert = (function() {
     assert.gleErrorCode = function(dbOrGLEDoc, code, msg) {
         var gle = dbOrGLEDoc instanceof DB ? dbOrGLEDoc.getLastErrorObj() : dbOrGLEDoc;
         if (!gle.err || gle.code != code) {
-            if (typeof(msg) == "function")
+            if (typeof (msg) == "function")
                 msg = msg(gle);
             doassert(_buildAssertionMessage(
                 msg,
@@ -1004,7 +999,7 @@ assert = (function() {
     assert.gleErrorRegex = function(dbOrGLEDoc, regex, msg) {
         var gle = dbOrGLEDoc instanceof DB ? dbOrGLEDoc.getLastErrorObj() : dbOrGLEDoc;
         if (!gle.err || !regex.test(gle.err)) {
-            if (typeof(msg) == "function")
+            if (typeof (msg) == "function")
                 msg = msg(gle);
             doassert(_buildAssertionMessage(
                 msg,
