@@ -12,14 +12,10 @@
  *     name {string}: name for this test
  *     verbose {number}: the verbosity for the mongos
  *     chunkSize {number}: the chunk size to use as configuration for the cluster
- *     nopreallocj {boolean|number}:
  *
  *     mongos {number|Object|Array.<Object>}: number of mongos or mongos
  *       configuration object(s)(*). @see MongoRunner.runMongos
  *
- *     mongosWaitsForKeys {boolean}: if true, wait for mongos to discover keys from the config
- *       server and to start sending cluster times.
- *
  *     rs {Object|Array.<Object>}: replica set configuration object. Can
  *       contain:
  *       {
@@ -54,7 +50,6 @@
  *          options take precedence.
  *
  *     other: {
- *       nopreallocj: same as above
  *       rs: same as above
  *       chunkSize: same as above
  *       keyFile {string}: the location of the keyFile
@@ -73,6 +68,9 @@
  *          enableBalancer setting
  *       manualAddShard {boolean}: shards will not be added if true.
  *
+ *       migrationLockAcquisitionMaxWaitMS {number}: number of milliseconds to acquire the migration
+ *          lock.
+ *
  *       useBridge {boolean}: If true, then a mongobridge process is started for each node in the
  *          sharded cluster. Defaults to false.
  *
@@ -89,6 +87,7 @@
  *       useHostname {boolean}: if true, use hostname of machine,
  *         otherwise use localhost
  *       numReplicas {number}
+ *
  *     }
  *   }
  *
@@ -103,7 +102,6 @@
  * configRS - If the config servers are a replset, this will contain the config ReplSetTest object
  */
 var ShardingTest = function(params) {
-
     if (!(this instanceof ShardingTest)) {
         return new ShardingTest(params);
     }
@@ -218,8 +216,12 @@ var ShardingTest = function(params) {
         if (!otherParams.enableBalancer) {
             self.stopBalancer();
         }
+
         if (!otherParams.enableAutoSplit) {
             self.disableAutoSplit();
+        } else if (!otherParams.enableBalancer) {
+            // Turn on autoSplit since disabling balancer also turns auto split off.
+            self.enableAutoSplit();
         }
     }
 
@@ -289,16 +291,16 @@ var ShardingTest = function(params) {
             countDBsFound++;
             printjson(db);
         });
-        throw Error("couldn't find dbname: " + dbname + " in config.databases. Total DBs: " +
-                    countDBsFound);
+        throw Error("couldn't find dbname: " + dbname +
+                    " in config.databases. Total DBs: " + countDBsFound);
     };
 
     this.getNonPrimaries = function(dbname) {
         var x = this.config.databases.findOne({_id: dbname});
         if (!x) {
             this.config.databases.find().forEach(printjson);
-            throw Error("couldn't find dbname: " + dbname + " total: " +
-                        this.config.databases.count());
+            throw Error("couldn't find dbname: " + dbname +
+                        " total: " + this.config.databases.count());
         }
 
         return this.config.shards.find({_id: {$ne: x.primary}}).map(z => z._id);
@@ -331,8 +333,8 @@ var ShardingTest = function(params) {
             }
         }
 
-        throw Error("can't find server connection for db '" + dbname + "'s primary shard: " +
-                    tojson(primaryShard));
+        throw Error("can't find server connection for db '" + dbname +
+                    "'s primary shard: " + tojson(primaryShard));
     };
 
     this.normalize = function(x) {
@@ -443,6 +445,12 @@ var ShardingTest = function(params) {
         throw _getErrorWithCode(res, "command " + tojson(cmd) + " failed: " + tojson(res));
     };
 
+    this.forEachConnection = function(fn) {
+        this._connections.forEach(function(conn) {
+            fn(conn);
+        });
+    };
+
     this.printChangeLog = function() {
         this.config.changelog.find().forEach(function(z) {
             var msg = z.server + "\t" + z.time + "\t" + z.what;
@@ -702,7 +710,7 @@ var ShardingTest = function(params) {
         }
 
         var result;
-        for (var i = 0; i < 10; i++) {
+        for (var i = 0; i < 5; i++) {
             var otherShard = this.getOther(this.getPrimaryShard(dbName)).name;
             result = this.s.adminCommand(
                 {movechunk: c, find: move, to: otherShard, _waitForDelete: waitForDelete});
@@ -860,9 +868,9 @@ var ShardingTest = function(params) {
         }
 
         if (arguments.length >= 3) {
-            if (typeof(beforeRestartCallback) !== "function") {
+            if (typeof (beforeRestartCallback) !== "function") {
                 throw new Error("beforeRestartCallback must be a function but was of type " +
-                                typeof(beforeRestartCallback));
+                                typeof (beforeRestartCallback));
             }
             beforeRestartCallback();
         }
@@ -1040,6 +1048,13 @@ var ShardingTest = function(params) {
         return false;
     }
 
+    /**
+     * Runs a find on the namespace to force a refresh of the node's catalog cache.
+     */
+    this.refreshCatalogCacheForNs = function(node, ns) {
+        node.getCollection(ns).findOne();
+    };
+
     /**
      * Returns if there is a new feature compatibility version for the "latest" version. This must
      * be manually changed if and when there is a new feature compatibility version.
@@ -1072,8 +1087,8 @@ var ShardingTest = function(params) {
     }
 
     // Allow specifying mixed-type options like this:
-    // { mongos : [ { noprealloc : "" } ],
-    //   config : [ { smallfiles : "" } ],
+    // { mongos : [ { bind_ip : "localhost" } ],
+    //   config : [ { nojournal : "" } ],
     //   shards : { rs : true, d : true } }
     if (Array.isArray(numShards)) {
         for (var i = 0; i < numShards.length; i++) {
@@ -1194,6 +1209,9 @@ var ShardingTest = function(params) {
         _allocatePortForShard = _allocatePortForMongos = allocatePort;
     }
 
+    otherParams.migrationLockAcquisitionMaxWaitMS =
+        otherParams.migrationLockAcquisitionMaxWaitMS || 30000;
+
     // Start the MongoD servers (shards)
     for (var i = 0; i < numShards; i++) {
         if (otherParams.rs || otherParams["rs" + i] || startShardsAsRS) {
@@ -1201,7 +1219,6 @@ var ShardingTest = function(params) {
 
             var rsDefaults = {
                 useHostname: otherParams.useHostname,
-                noJournalPrealloc: otherParams.nopreallocj,
                 oplogSize: 16,
                 shardsvr: '',
                 pathOpts: Object.merge(pathOpts, {shard: i}),
@@ -1243,6 +1260,10 @@ var ShardingTest = function(params) {
                 rsDefaults = Object.merge(rsDefaults, otherParams.shardOptions);
             }
 
+            rsDefaults.setParameter = rsDefaults.setParameter || {};
+            rsDefaults.setParameter.migrationLockAcquisitionMaxWaitMS =
+                otherParams.migrationLockAcquisitionMaxWaitMS;
+
             var rsSettings = rsDefaults.settings;
             delete rsDefaults.settings;
 
@@ -1290,13 +1311,16 @@ var ShardingTest = function(params) {
         } else {
             var options = {
                 useHostname: otherParams.useHostname,
-                noJournalPrealloc: otherParams.nopreallocj,
                 pathOpts: Object.merge(pathOpts, {shard: i}),
                 dbpath: "$testName$shard",
                 shardsvr: '',
                 keyFile: keyFile
             };
 
+            options.setParameter = options.setParameter || {};
+            options.setParameter.migrationLockAcquisitionMaxWaitMS =
+                otherParams.migrationLockAcquisitionMaxWaitMS;
+
             if (jsTestOptions().shardMixedBinVersions) {
                 if (!otherParams.shardOptions) {
                     otherParams.shardOptions = {};
@@ -1401,7 +1425,6 @@ var ShardingTest = function(params) {
         // Ensure that journaling is always enabled for config servers.
         journal: "",
         configsvr: "",
-        noJournalPrealloc: otherParams.nopreallocj,
         storageEngine: "wiredTiger",
     };
 
@@ -1463,7 +1486,7 @@ var ShardingTest = function(params) {
     const configRS = this.configRS;
     if (_hasNewFeatureCompatibilityVersion() && _isMixedVersionCluster()) {
         function setFeatureCompatibilityVersion() {
-            assert.commandWorked(csrsPrimary.adminCommand({setFeatureCompatibilityVersion: '3.6'}));
+            assert.commandWorked(csrsPrimary.adminCommand({setFeatureCompatibilityVersion: '4.0'}));
 
             // Wait for the new featureCompatibilityVersion to propagate to all nodes in the CSRS
             // to ensure that older versions of mongos can successfully connect.
@@ -1563,8 +1586,15 @@ var ShardingTest = function(params) {
     // the instances and then log them out again.
     if (keyFile) {
         authutil.asCluster(this._mongos, keyFile, _configureCluster);
+    } else if (mongosOptions[0] && mongosOptions[0].keyFile) {
+        authutil.asCluster(this._mongos, mongosOptions[0].keyFile, _configureCluster);
     } else {
         _configureCluster();
+        // Ensure that all config server nodes are up to date with any changes made to balancer
+        // settings before adding shards to the cluster. This prevents shards, which read
+        // config.settings with readPreference 'nearest', from accidentally fetching stale values
+        // from secondaries that aren't up-to-date.
+        this.configRS.awaitLastOpCommitted();
     }
 
     try {
@@ -1602,23 +1632,6 @@ var ShardingTest = function(params) {
         jsTest.authenticateNodes(this._mongos);
     }
 
-    // Mongos does not block for keys from the config servers at startup, so it may not initially
-    // return cluster times. If mongosWaitsForKeys is set, block until all mongos servers have found
-    // the keys and begun to send cluster times. Retry every 500 milliseconds and timeout after 60
-    // seconds.
-    if (params.mongosWaitsForKeys) {
-        assert.soon(function() {
-            for (let i = 0; i < numMongos; i++) {
-                const res = self._mongos[i].adminCommand({isMaster: 1});
-                if (!res.hasOwnProperty("$clusterTime")) {
-                    print("Waiting for mongos #" + i + " to start sending cluster times.");
-                    return false;
-                }
-            }
-            return true;
-        }, "waiting for all mongos servers to return cluster times", 60 * 1000, 500);
-    }
-
     // Ensure that the sessions collection exists so jstests can run things with
     // logical sessions and test them. We do this by forcing an immediate cache refresh
     // on the config server, which auto-shards the collection for the cluster.
@@ -1640,7 +1653,6 @@ var ShardingTest = function(params) {
             }
         }
     }
-
 };
 
 // Stub for a hook to check that collection UUIDs are consistent across shards and the config
