@@ -396,6 +396,8 @@ std::unique_ptr<SSLManagerInterface> SSLManagerInterface::create(const SSLParams
 
 namespace {
 
+StatusWith<std::vector<std::string>> getSubjectAlternativeNames(PCCERT_CONTEXT cert);
+
 SSLManagerWindows::SSLManagerWindows(const SSLParams& params, bool isServer)
     : _weakValidation(params.sslWeakCertificateValidation),
       _allowInvalidCertificates(params.sslAllowInvalidCertificates),
@@ -432,6 +434,13 @@ SSLManagerWindows::SSLManagerWindows(const SSLParams& params, bool isServer)
                                      &subjectName,
                                      &_sslConfiguration.serverCertificateExpirationDate));
             uassertStatusOK(_sslConfiguration.setServerSubjectName(std::move(subjectName)));
+
+            auto swSans = getSubjectAlternativeNames(_serverCertificates[0]);
+            const bool hasSan = swSans.isOK() && (0 != swSans.getValue().size());
+            if (!hasSan) {
+                warning() << "Server certificate has no compatible Subject Alternative Name. "
+                             "This may prevent TLS clients from connecting";
+            }
         }
 
         // Monitor the server certificate's expiration
